@@ -274,7 +274,7 @@ gnc_tree_view_init (GncTreeView *view, void *data)
     /* Create the last column which contains the column selection
      * widget.  gnc_tree_view_add_text_column will do most of the
      * work. */
-    icon = gtk_image_new_from_icon_name ("go-down", GTK_ICON_SIZE_SMALL_TOOLBAR);
+    icon = gtk_image_new_from_icon_name ("pan-down-symbolic", GTK_ICON_SIZE_SMALL_TOOLBAR);
 
     priv->column_menu_icon_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_box_set_homogeneous (GTK_BOX(priv->column_menu_icon_box), FALSE);
@@ -288,9 +288,6 @@ gnc_tree_view_init (GncTreeView *view, void *data)
 
     gtk_widget_show_all (priv->column_menu_icon_box);
 
-    g_signal_connect (G_OBJECT(icon), "draw",
-                      G_CALLBACK(gnc_draw_arrow_cb), GINT_TO_POINTER(1));
-
     column = gnc_tree_view_add_text_column (view, NULL, NULL, NULL, NULL,
                                             -1, -1, NULL);
     g_object_set (G_OBJECT(column),
@@ -1167,6 +1164,8 @@ void gnc_tree_view_save_state (GncTreeView *view)
         else if (g_key_file_has_key (state_file, priv->state_section, STATE_KEY_COLUMN_ORDER, NULL))
             g_key_file_remove_key (state_file, priv->state_section, STATE_KEY_COLUMN_ORDER, NULL);
 
+        g_strfreev (col_order);
+
 
         // ENTER("view %p, wanted %s", view, wanted);
         column_list = gtk_tree_view_get_columns (GTK_TREE_VIEW(view));
@@ -2119,6 +2118,7 @@ gnc_tree_view_add_numeric_column (GncTreeView *view,
 {
     GtkTreeViewColumn *column;
     GtkCellRenderer *renderer;
+    gfloat alignment = 1.0;
 
     column = gnc_tree_view_add_text_column (view, column_title, pref_name,
                                             NULL, sizing_text, model_data_column,
@@ -2127,9 +2127,12 @@ gnc_tree_view_add_numeric_column (GncTreeView *view,
 
     renderer = gnc_tree_view_column_get_renderer (column);
 
-    /* Right align the column title and data */
-    g_object_set (G_OBJECT(column), "alignment",   1.0, NULL);
-    g_object_set (G_OBJECT(renderer), "xalign",   1.0, NULL);
+    /* Right align the column title and data for both ltr and rtl */
+    if (gtk_widget_get_direction (GTK_WIDGET(view)) == GTK_TEXT_DIR_RTL)
+        alignment = 0.0;
+
+    g_object_set (G_OBJECT(column), "alignment", alignment, NULL);
+    g_object_set (G_OBJECT(renderer), "xalign", alignment, NULL);
 
     /* Change the text color */
     if (model_color_column != GNC_TREE_VIEW_COLUMN_COLOR_NONE)
