@@ -32,6 +32,7 @@
 #include "gnc-ui.h"
 #include "gnc-gui-query.h"
 #include "gnc-ui-util.h"
+#include <gnc-glib-utils.h>
 #include "qof.h"
 #include "gnc-date.h"
 #include "gnc-date-edit.h"
@@ -53,6 +54,9 @@
 #include "dialog-transfer.h"
 #include "dialog-print-check.h"
 #include "gnc-general-search.h"
+#include <qoflog.h>
+
+static const QofLogModule log_module = G_LOG_DOMAIN;
 
 #define DIALOG_PAYMENT_CM_CLASS "payment-dialog"
 
@@ -201,6 +205,7 @@ void gnc_payment_acct_tree_row_activated_cb (GtkWidget *widget, GtkTreePath *pat
         GtkTreeViewColumn *column, PaymentWindow *pw);
 void gnc_payment_leave_amount_cb (GtkWidget *widget, GdkEventFocus *event,
                                   PaymentWindow *pw);
+void gnc_payment_activate_amount_cb (GtkWidget *widget, PaymentWindow *pw);
 void gnc_payment_window_fill_docs_list (PaymentWindow *pw);
 
 
@@ -221,6 +226,7 @@ gnc_payment_window_check_payment (PaymentWindow *pw)
     gboolean enable_xfer_acct = TRUE;
     gboolean allow_payment = TRUE;
     GtkTreeSelection *selection;
+    gint c_result, d_result;
 
     if (!pw)
         return FALSE;
@@ -242,11 +248,24 @@ gnc_payment_window_check_payment (PaymentWindow *pw)
         goto update_cleanup;
     }
 
+    /* Verify the credit / debit amounts are valid */
+    d_result = gnc_amount_edit_expr_is_valid (GNC_AMOUNT_EDIT(pw->amount_debit_edit),
+                                              &amount_deb, FALSE, NULL);
+
+    c_result = gnc_amount_edit_expr_is_valid (GNC_AMOUNT_EDIT(pw->amount_credit_edit),
+                                              &amount_cred, FALSE, NULL);
+
+    if ((d_result == 1) || (c_result == 1))
+    {
+        conflict_msg = _("There is a problem with the Payment or Refund amount.");
+        allow_payment = FALSE;
+        goto update_cleanup;
+    }
+
     /* Test the total amount */
-    amount_deb  = gnc_amount_edit_get_amount (GNC_AMOUNT_EDIT (pw->amount_debit_edit));
-    amount_cred = gnc_amount_edit_get_amount (GNC_AMOUNT_EDIT (pw->amount_credit_edit));
     pw->amount_tot = gnc_numeric_sub (amount_cred, amount_deb,
-                                      gnc_commodity_get_fraction (xaccAccountGetCommodity (pw->post_acct)),
+                                      gnc_commodity_get_fraction (
+                                      xaccAccountGetCommodity (pw->post_acct)),
                                       GNC_HOW_RND_ROUND_HALF_UP);
 
     if (gnc_numeric_check (pw->amount_tot) || gnc_numeric_zero_p (pw->amount_tot))
@@ -1104,24 +1123,40 @@ gnc_payment_leave_amount_cb (G_GNUC_UNUSED GtkWidget *widget,
                              G_GNUC_UNUSED GdkEventFocus *event,
                              PaymentWindow *pw)
 {
-    gnc_numeric amount_deb, amount_cred, amount_tot;
+    gboolean d_payment_ok = FALSE;
+    gboolean c_payment_ok = FALSE;
 
     if (! pw->amount_credit_edit || ! pw->amount_debit_edit)
         return;
 
-    /* If both credit and debit amount are entered, simplify it to either one */
-    amount_deb  = gnc_amount_edit_get_amount (GNC_AMOUNT_EDIT (pw->amount_debit_edit));
-    amount_cred = gnc_amount_edit_get_amount (GNC_AMOUNT_EDIT (pw->amount_credit_edit));
-    amount_tot = gnc_numeric_sub (amount_cred, amount_deb,
-                                  gnc_commodity_get_fraction (xaccAccountGetCommodity (pw->post_acct)),
-                                  GNC_HOW_RND_ROUND_HALF_UP);
+    c_payment_ok = gnc_amount_edit_evaluate (GNC_AMOUNT_EDIT(pw->amount_credit_edit), NULL);
+    d_payment_ok = gnc_amount_edit_evaluate (GNC_AMOUNT_EDIT(pw->amount_debit_edit), NULL);
 
-    gnc_ui_payment_window_set_amount (pw, amount_tot);
+    if (c_payment_ok && d_payment_ok)
+    {
+        gnc_numeric amount_deb, amount_cred, amount_tot;
 
+        /* If both credit and debit amount are entered, simplify it to either one */
+        amount_deb  = gnc_amount_edit_get_amount (GNC_AMOUNT_EDIT (pw->amount_debit_edit));
+        amount_cred = gnc_amount_edit_get_amount (GNC_AMOUNT_EDIT (pw->amount_credit_edit));
+        amount_tot = gnc_numeric_sub (amount_cred, amount_deb,
+                                      gnc_commodity_get_fraction (
+                                      xaccAccountGetCommodity (pw->post_acct)),
+                                      GNC_HOW_RND_ROUND_HALF_UP);
+
+        gnc_ui_payment_window_set_amount (pw, amount_tot);
+    }
     /* Reflect if the payment could complete now */
     gnc_payment_window_check_payment (pw);
 }
 
+void
+gnc_payment_activate_amount_cb (G_GNUC_UNUSED GtkWidget *widget,
+                                PaymentWindow *pw)
+{
+      gnc_payment_leave_amount_cb (NULL, NULL, pw);
+}
+
 /* Select the list of accounts to show in the tree */
 static void
 gnc_payment_set_account_types (GncTreeViewAccount *tree)
@@ -1294,6 +1329,10 @@ new_payment_window (GtkWindow *parent, QofBook *book, InitialPaymentInfo *tx_inf
                      "focus-out-event",
                      G_CALLBACK(gnc_payment_leave_amount_cb), pw);
 
+    g_signal_connect(G_OBJECT(pw->amount_debit_edit),
+                     "activate",
+                     G_CALLBACK(gnc_payment_activate_amount_cb), pw);
+
     pw->amount_credit_edit = gnc_amount_edit_new ();
     gnc_amount_edit_set_evaluate_on_enter (GNC_AMOUNT_EDIT (pw->amount_credit_edit),
                                            TRUE);
@@ -1302,6 +1341,10 @@ new_payment_window (GtkWindow *parent, QofBook *book, InitialPaymentInfo *tx_inf
                      "focus-out-event",
                      G_CALLBACK(gnc_payment_leave_amount_cb), pw);
 
+    g_signal_connect(G_OBJECT(pw->amount_credit_edit),
+                     "activate",
+                     G_CALLBACK(gnc_payment_activate_amount_cb), pw);
+
     box = GTK_WIDGET (gtk_builder_get_object (builder, "date_box"));
     pw->date_edit = gnc_date_edit_new (time(NULL), FALSE, FALSE);
     gtk_box_pack_start (GTK_BOX (box), pw->date_edit, TRUE, TRUE, 0);
@@ -1522,7 +1565,7 @@ gboolean gnc_ui_payment_is_customer_payment(const Transaction *txn)
     {
         /* Transaction isn't valid for a payment, just return the default
          * Calling code will have to handle this situation properly */
-        g_message("No asset splits in txn \"%s\"; cannot use this for assigning a payment.",
+        PINFO("No asset splits in txn \"%s\"; cannot use this for assigning a payment.",
                   xaccTransGetDescription(txn));
         return result;
     }
@@ -1530,7 +1573,7 @@ gboolean gnc_ui_payment_is_customer_payment(const Transaction *txn)
     assetaccount_split = xaccTransGetFirstPaymentAcctSplit(txn);
     amount = xaccSplitGetValue(assetaccount_split);
     result = gnc_numeric_positive_p(amount); // positive amounts == customer
-    //g_message("Amount=%s", gnc_numeric_to_string(amount));
+    //PINFO("Amount=%s", gnc_numeric_to_string(amount));
     return result;
 }
 
@@ -1579,7 +1622,7 @@ static Split *select_payment_split (GtkWindow *parent, Transaction *txn)
                                          _("The selected transaction doesn't have splits that can be assigned as a payment"));
         gtk_dialog_run (GTK_DIALOG(dialog));
         gtk_widget_destroy (dialog);
-        g_message("No asset splits in txn \"%s\"; cannot use this for assigning a payment.",
+        PINFO("No asset splits in txn \"%s\"; cannot use this for assigning a payment.",
                   xaccTransGetDescription(txn));
         return NULL;
     }
@@ -1688,10 +1731,12 @@ static GList *select_txn_lots (GtkWindow *parent, Transaction *txn, Account **po
         }
     }
 
+    g_list_free (post_splits);
+
     /* If the txn has both APAR splits linked to a business lot and
      * splits that are not, issue a warning some will be discarded.
      */
-    if (has_no_lot_apar_splits && (g_list_length (txn_lots) > 0))
+    if (has_no_lot_apar_splits && gnc_list_length_cmp (txn_lots, 0))
     {
         GtkWidget *dialog;
         char *split_str = g_strdup ("");
@@ -1726,6 +1771,8 @@ static GList *select_txn_lots (GtkWindow *parent, Transaction *txn, Account **po
         g_free (split_str);
     }
 
+    g_list_free (no_lot_post_splits);
+
     return txn_lots;
 }
 
