@@ -150,11 +150,7 @@ gnc_reconcile_view_tooltip_cb (GNCQueryView *qview, gint x, gint y,
 
             if (keyboard_mode == FALSE)
             {
-#if GTK_CHECK_VERSION(3,20,0)
                 GdkSeat *seat;
-#else
-                GdkDeviceManager *device_manager;
-#endif
                 GdkDevice *pointer;
                 GtkWindow *tip_win = NULL;
                 GdkWindow *parent_window;
@@ -162,13 +158,9 @@ gnc_reconcile_view_tooltip_cb (GNCQueryView *qview, gint x, gint y,
 
                 parent_window = gtk_widget_get_parent_window (GTK_WIDGET (qview));
 
-#if GTK_CHECK_VERSION(3,20,0)
                 seat = gdk_display_get_default_seat (gdk_window_get_display (parent_window));
                 pointer = gdk_seat_get_pointer (seat);
-#else
-                device_manager = gdk_display_get_device_manager (gdk_window_get_display (parent_window));
-                pointer = gdk_device_manager_get_client_pointer (device_manager);
-#endif
+
                 gdk_window_get_device_position (parent_window, pointer, &cur_x, &cur_y, NULL);
 
                 gdk_window_get_origin (parent_window, &root_x, &root_y);
@@ -189,12 +181,7 @@ gnc_reconcile_view_tooltip_cb (GNCQueryView *qview, gint x, gint y,
 
                 if (GTK_IS_WINDOW (tip_win))
                 {
-#if GTK_CHECK_VERSION(3,22,0)
                     GdkMonitor *mon;
-#else
-                    GdkScreen *screen;
-                    gint monitor_num;
-#endif
                     GdkRectangle monitor;
                     GtkRequisition requisition;
                     gint x, y;
@@ -204,14 +191,9 @@ gnc_reconcile_view_tooltip_cb (GNCQueryView *qview, gint x, gint y,
                     x = root_x + cur_x + 10;
                     y = root_y + cur_y + 10;
 
-#if GTK_CHECK_VERSION(3,22,0)
-                    mon = gdk_display_get_monitor_at_point (gdk_display_get_default(), x, y);
+                    mon = gdk_display_get_monitor_at_point (gdk_window_get_display (parent_window), x, y);
                     gdk_monitor_get_geometry (mon, &monitor);
-#else
-                    screen = gtk_widget_get_screen (GTK_WIDGET (qview));
-                    monitor_num = gdk_screen_get_monitor_at_point (screen, x, y);
-                    gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
-#endif
+
                     if (x + requisition.width > monitor.x + monitor.width)
                         x -= x - (monitor.x + monitor.width) + requisition.width;
                     else if (x < monitor.x)
@@ -331,6 +313,7 @@ gnc_reconcile_view_new (Account *account, GNCReconcileViewType type,
     GList            *accounts = NULL;
     GList            *splits;
     Query            *query;
+    QofNumericMatch   sign;
 
     g_return_val_if_fail (account, NULL);
     g_return_val_if_fail ((type == RECLIST_DEBIT) ||
@@ -363,15 +346,11 @@ gnc_reconcile_view_new (Account *account, GNCReconcileViewType type,
 
     g_list_free (accounts);
 
-    /* limit the matches to CREDITs and DEBITs only, depending on the type */
-    if (type == RECLIST_CREDIT)
-        xaccQueryAddValueMatch(query, gnc_numeric_zero (),
-                               QOF_NUMERIC_MATCH_CREDIT,
-                               QOF_COMPARE_GTE, QOF_QUERY_AND);
-    else
-        xaccQueryAddValueMatch(query, gnc_numeric_zero (),
-                               QOF_NUMERIC_MATCH_DEBIT,
-                               QOF_COMPARE_GTE, QOF_QUERY_AND);
+    sign = (type == RECLIST_CREDIT) ?
+        QOF_NUMERIC_MATCH_CREDIT : QOF_NUMERIC_MATCH_DEBIT;
+
+    xaccQueryAddNumericMatch (query, gnc_numeric_zero (), sign, QOF_COMPARE_GTE,
+                              QOF_QUERY_AND, SPLIT_AMOUNT, NULL);
 
     /* limit the matches only to Cleared and Non-reconciled splits */
     xaccQueryAddClearedMatch (query, CLEARED_NO | CLEARED_CLEARED, QOF_QUERY_AND);
@@ -501,42 +480,8 @@ gnc_reconcile_view_class_init (GNCReconcileViewClass *klass)
 }
 
 
-static gboolean
-gnc_reconcile_view_toggle_split (GNCReconcileView *view, Split *split)
-{
-    Split *current;
-
-    g_return_val_if_fail (GNC_IS_RECONCILE_VIEW (view), FALSE);
-    g_return_val_if_fail (view->reconciled != NULL, FALSE);
-
-    current = g_hash_table_lookup (view->reconciled, split);
-
-    if (current == NULL)
-    {
-        g_hash_table_insert (view->reconciled, split, split);
-        return TRUE;
-    }
-    else
-    {
-        g_hash_table_remove (view->reconciled, split);
-        return FALSE;
-    }
-}
-
-
-/** Insert or remove a split from the list of splits to be reconciled
- *   (view->reconciled) so that all other splits in the same transaction
- *   for the account being reconciled (including children), are the same
- *   reconciliation state as the split that has been toggled.
- *
- *  @param view The view to use.
- *
- *  @param split The split to be inserted or removed
- *
- *  @param reconcile TRUE=insert, FALSE=remove
- */
 static void
-gnc_reconcile_view_rec_or_unrec_split (GNCReconcileView *view, Split *split, gboolean reconcile)
+gnc_reconcile_view_toggle_split (GNCReconcileView *view, Split *split)
 {
     Split *current;
 
@@ -545,103 +490,20 @@ gnc_reconcile_view_rec_or_unrec_split (GNCReconcileView *view, Split *split, gbo
 
     current = g_hash_table_lookup (view->reconciled, split);
 
-    if (current == NULL && reconcile)
+    if (current == NULL)
         g_hash_table_insert (view->reconciled, split, split);
-    if ((current != NULL) && (!reconcile))
+    else
         g_hash_table_remove (view->reconciled, split);
 }
 
 
-static void
-gnc_reconcile_view_toggle_children (Account *account, GNCReconcileView *view, Split *split, gboolean reconcile)
-{
-    GList       *child_accounts, *node;
-    Transaction *transaction;
-
-    /*
-     * Need to get all splits in this transaction and identify any that are
-     * in the same hierarchy as the account being reconciled (not necessarily
-     * the account this split is from.)
-     *
-     * For each of these splits set them to the same state as the split whose
-     * checkbox was toggled.
-     */
-    child_accounts = gnc_account_get_descendants (account);
-    child_accounts = g_list_prepend (child_accounts, account);
-    transaction = xaccSplitGetParent (split);
-    for (node = xaccTransGetSplitList (transaction); node; node = node->next)
-    {
-        Split *other_split;
-        Account *other_account;
-        GNCReconcileView *current_view;
-
-        GtkTreeModel *model;
-        GtkTreeIter   iter;
-        gboolean      valid;
-        gpointer      pointer;
-
-        other_split = node->data;
-        other_account = xaccSplitGetAccount (other_split);
-        if (other_split == split)
-            continue;
-        /* Check this 'other' account in in the same hierarchy */
-        if (!g_list_find (child_accounts, other_account))
-            continue;
-        /* Search our sibling view for this split first.  We search the
-         * sibling list first because that it where it is most likely to be.
-         */
-        current_view = view->sibling;
-        if (!gnc_query_view_item_in_view (GNC_QUERY_VIEW (current_view), other_split))
-        {
-            /* Not in the sibling view, try this view */
-            current_view = view;
-            if (!gnc_query_view_item_in_view (GNC_QUERY_VIEW (current_view), other_split))
-                /* We can't find it, nothing more I can do about it */
-                continue;
-        }
-
-        /* Found the other split. Toggle the reconciled check mark in the view... */
-        model = gtk_tree_view_get_model (GTK_TREE_VIEW (current_view));
-        valid = gtk_tree_model_get_iter_first (model, &iter);
-
-        while (valid)
-        {
-            // Walk through the list, reading each row
-            gtk_tree_model_get (model, &iter, REC_POINTER, &pointer, -1);
-
-            if(pointer == other_split)
-            {
-                gboolean toggled;
-                gtk_tree_model_get (model, &iter, REC_RECN, &toggled, -1);
-                if (toggled != reconcile)
-                    gtk_list_store_set (GTK_LIST_STORE (model), &iter, REC_RECN, reconcile, -1);
-                break;
-            }
-
-            valid = gtk_tree_model_iter_next (model, &iter);
-        }
-
-        /* ...and toggle its reconciled state in the internal hash */
-        gnc_reconcile_view_rec_or_unrec_split (current_view, other_split, reconcile);
-    }
-    g_list_free (child_accounts);
-}
-
-
 static void
 gnc_reconcile_view_toggle (GNCReconcileView *view, Split *split)
 {
-    gboolean include_children;
-    gboolean is_reconciled;
-
     g_return_if_fail (GNC_IS_RECONCILE_VIEW (view));
     g_return_if_fail (view->reconciled != NULL);
 
-    is_reconciled = gnc_reconcile_view_toggle_split (view, split);
-
-    include_children = xaccAccountGetReconcileChildrenStatus (view->account);
-    if (include_children)
-        gnc_reconcile_view_toggle_children (view->account, view, split, is_reconciled);
+    gnc_reconcile_view_toggle_split (view, split);
 
     g_signal_emit (G_OBJECT (view),
                    reconcile_view_signals[TOGGLE_RECONCILED], 0, split);
@@ -825,6 +687,7 @@ gnc_reconcile_view_finalize (GObject *object)
 {
     GNCReconcileView *view = GNC_RECONCILE_VIEW (object);
 
+    g_list_free (view->column_list);
     if (view->reconciled != NULL)
     {
         g_hash_table_destroy (view->reconciled);
@@ -901,6 +764,8 @@ void
 gnc_reconcile_view_refresh (GNCReconcileView *view)
 {
     GNCQueryView *qview;
+    GtkTreeSelection *selection;
+    GList *path_list, *node;
 
     g_return_if_fail (view != NULL);
     g_return_if_fail (GNC_IS_RECONCILE_VIEW (view));
@@ -908,6 +773,18 @@ gnc_reconcile_view_refresh (GNCReconcileView *view)
     qview = GNC_QUERY_VIEW (view);
     gnc_query_view_refresh (qview);
 
+    /* Ensure last selected split, if any, can be seen */
+    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (qview));
+    path_list = gtk_tree_selection_get_selected_rows (selection, NULL);
+    node = g_list_last (path_list);
+    if (node)
+    {
+        GtkTreePath *tree_path = node->data;
+        gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (qview),
+                                      tree_path, NULL, FALSE, 0.0, 0.0);
+    }
+    g_list_free_full (path_list, (GDestroyNotify) gtk_tree_path_free);
+
     /* Now verify that everything in the reconcile hash is still in qview */
     if (view->reconciled)
         g_hash_table_foreach (view->reconciled, grv_refresh_helper, view);
