@@ -46,7 +46,7 @@
 
 #define GNCIMPORT_DESC    "desc"
 #define GNCIMPORT_MEMO    "memo"
-#define GNCIMPORT_PAYEE    "payee"
+#define GNCIMPORT_PAYEE   "payee"
 
 /********************************************************************\
  *   Constants                                                      *
@@ -54,13 +54,6 @@
 
 static QofLogModule log_module = GNC_MOD_IMPORT;
 
-/********************************************************************\
- *   Constants, should ideally be defined a user preference dialog  *
-\********************************************************************/
-
-static const int MATCH_DATE_THRESHOLD = 4; /*within 4 days*/
-static const int MATCH_DATE_NOT_THRESHOLD = 14;
-
 /********************************************************************\
  *   Forward declared prototypes                                    *
 \********************************************************************/
@@ -102,6 +95,9 @@ struct _transactioninfo
 
     /* Reference id to link gnc transaction to external object. E.g. aqbanking job id. */
     guint32 ref_id;
+
+    /* When updating a matched transaction, append Description and Notes instead of replacing */
+    gboolean append_text;
 };
 
 /* Some simple getters and setters for the above data types. */
@@ -247,6 +243,15 @@ gnc_import_TransInfo_set_ref_id (GNCImportTransInfo *info,
 }
 
 
+void
+gnc_import_TransInfo_set_append_text (GNCImportTransInfo *info,
+                                      gboolean append_text)
+{
+    g_assert (info);
+    info->append_text = append_text;
+}
+
+
 Split *
 gnc_import_MatchInfo_get_split (const GNCImportMatchInfo * info)
 {
@@ -622,6 +627,8 @@ matchmap_store_destination (GncImportMatchMap *matchmap,
 void split_find_match (GNCImportTransInfo * trans_info,
                               Split * split,
                               gint display_threshold,
+                              gint date_threshold,
+                              gint date_not_threshold,
                               double fuzzy_amount_difference)
 {
     /* DEBUG("Begin"); */
@@ -691,12 +698,12 @@ void split_find_match (GNCImportTransInfo * trans_info,
             prob = prob + 3;
             /*DEBUG("heuristics:  probability + 3 (date)");*/
         }
-        else if (datediff_day <= MATCH_DATE_THRESHOLD)
+        else if (datediff_day <= date_threshold)
         {
             prob = prob + 2;
             /*DEBUG("heuristics:  probability + 2 (date)");*/
         }
-        else if (datediff_day > MATCH_DATE_NOT_THRESHOLD)
+        else if (datediff_day > date_not_threshold)
         {
             /* Extra penalty if that split lies awfully far away from
                the given one. */
@@ -832,6 +839,77 @@ void split_find_match (GNCImportTransInfo * trans_info,
 /***********************************************************************
  */
 
+/* append the imported transaction description to the matched transaction description */
+static void
+desc_append (Transaction* selected_match_trans, Transaction* imp_trans)
+{
+    gchar* tmp = g_strconcat( xaccTransGetDescription (selected_match_trans),
+                              "|",
+                              xaccTransGetDescription (imp_trans),
+                              NULL);
+    xaccTransSetDescription (selected_match_trans, tmp);
+    g_free (tmp);
+}
+
+/* append the imported transaction notes to the matched transaction notes */
+static void
+notes_append (Transaction* selected_match_trans, Transaction* imp_trans)
+{
+    gchar* tmp = g_strconcat (xaccTransGetNotes (selected_match_trans),
+                              "|",
+                              xaccTransGetNotes (imp_trans),
+                              NULL);
+    xaccTransSetNotes (selected_match_trans, tmp);
+    g_free (tmp);
+}
+
+/* Append or replace transaction description and notes
+ * depending on the Append checkbox
+ */
+static void
+update_desc_and_notes (const GNCImportTransInfo* trans_info)
+{
+    GNCImportMatchInfo* selected_match =
+            gnc_import_TransInfo_get_selected_match (trans_info);
+    Transaction* imp_trans = gnc_import_TransInfo_get_trans (trans_info);
+
+    if (trans_info->append_text)
+    {
+        gchar* desc_imported = g_utf8_normalize (xaccTransGetDescription (
+            imp_trans), -1, G_NORMALIZE_ALL);
+        gchar* desc_matched = g_utf8_normalize (xaccTransGetDescription (
+            selected_match->trans), -1, G_NORMALIZE_ALL);
+        gchar* note_imported = g_utf8_normalize (xaccTransGetNotes (
+                imp_trans), -1, G_NORMALIZE_ALL);
+        gchar* note_matched = g_utf8_normalize (xaccTransGetNotes (
+            selected_match->trans), -1, G_NORMALIZE_ALL);
+
+        // Append if desc_imported not already in desc_matched
+        if (g_utf8_strlen (desc_imported, -1) > g_utf8_strlen (desc_matched, -1) ||
+            !strstr (desc_matched, desc_imported))
+            desc_append (selected_match->trans, imp_trans);
+
+        // Append if note_imported not already in note_matched
+        if (g_utf8_strlen (note_imported, -1) > g_utf8_strlen (note_matched, -1) ||
+            !strstr (note_matched, note_imported))
+            notes_append (selected_match->trans, imp_trans);
+
+        g_free(desc_imported);
+        g_free(desc_matched);
+        g_free(note_imported);
+        g_free(note_matched);
+    }
+    else
+    {
+        // replace the matched transaction description with the imported transaction description
+        xaccTransSetDescription (selected_match->trans,
+                                 xaccTransGetDescription (imp_trans));
+        // replace the matched transaction notes with the imported transaction notes
+        xaccTransSetNotes (selected_match->trans,
+                           xaccTransGetNotes (imp_trans));
+    }
+}
+
 /** /brief -- Processes one match
    according to its selected action.  */
 gboolean
@@ -949,13 +1027,7 @@ gnc_import_process_trans_item (GncImportMatchMap *matchmap,
                    to balance the transaction */
             }
 
-            xaccTransSetDescription(selected_match->trans,
-                                    xaccTransGetDescription(
-                                        gnc_import_TransInfo_get_trans(trans_info)));
-
-            xaccTransSetNotes(selected_match->trans,
-                                    xaccTransGetNotes(
-                                        gnc_import_TransInfo_get_trans(trans_info)));
+            update_desc_and_notes( trans_info);
 
             if (xaccSplitGetReconcile(selected_match->split) == NREC)
             {
@@ -1096,24 +1168,6 @@ static gint check_trans_online_id(Transaction *trans1, void *user_data)
     }
 }
 
-static gint collect_trans_online_id(Transaction *trans, void *user_data)
-{
-    Split *split;
-    GHashTable *id_hash = user_data;
-    int i=0;
-    
-    const gchar* online_id = gnc_import_get_trans_online_id (trans);
-    if (online_id)
-        g_hash_table_add (id_hash, (void*) online_id);
-
-    for (GList *splits = xaccTransGetSplitList (trans); splits; splits = splits->next)
-    {
-        if (gnc_import_split_has_online_id (splits->data))
-            g_hash_table_add(id_hash, (void*) gnc_import_get_split_online_id (splits->data));
-    }
-    return 0;
-}
-
 /** Checks whether the given transaction's online_id already exists in
   its parent account. */
 gboolean gnc_import_exists_online_id (Transaction *trans, GHashTable* acct_id_hash)
@@ -1129,14 +1183,19 @@ gboolean gnc_import_exists_online_id (Transaction *trans, GHashTable* acct_id_ha
     // No online id, no point in continuing. We'd crash if we tried.
     if (!gnc_import_get_split_online_id (source_split))
         return FALSE;
-    // Create a hash per account of a hash of all transactions IDs. Then the test below will be fast if
+    // Create a hash per account of a hash of all split IDs. Then the test below will be fast if
     // we have many transactions to import.
     dest_acct = xaccSplitGetAccount (source_split);
     if (!g_hash_table_contains (acct_id_hash, dest_acct))
     {
         GHashTable* new_hash = g_hash_table_new (g_str_hash, g_str_equal);
+        GList* split_list = xaccAccountGetSplitList(dest_acct);
         g_hash_table_insert (acct_id_hash, dest_acct, new_hash);
-        xaccAccountForEachTransaction (dest_acct, collect_trans_online_id, new_hash);
+        for (;split_list;split_list=split_list->next)
+        {
+            if (gnc_import_split_has_online_id (split_list->data))
+                g_hash_table_add (new_hash, (void*) gnc_import_get_split_online_id (split_list->data));
+        }
     }
     online_id_exists = g_hash_table_contains (g_hash_table_lookup (acct_id_hash, dest_acct),
                                               gnc_import_get_split_online_id (source_split));
