@@ -40,6 +40,7 @@
 #include "import-main-matcher.h"
 
 #include "dialog-utils.h"
+#include "gnc-glib-utils.h"
 #include "gnc-ui.h"
 #include "gnc-ui-util.h"
 #include "gnc-engine.h"
@@ -72,6 +73,7 @@ struct _main_matcher_info
     GtkTreeViewColumn       *memo_column;
     GtkWidget               *show_account_column;
     GtkWidget               *show_matched_info;
+    GtkWidget               *append_text; // Update+Clear: Append import Desc/Notes to matched Desc/Notes
     GtkWidget               *reconcile_after_close;
     gboolean add_toggled;     // flag to indicate that add has been toggled to stop selection
     gint id;
@@ -444,7 +446,22 @@ resolve_conflicts (GNCImportMainMatcher *info)
 void
 gnc_gen_trans_list_show_all (GNCImportMainMatcher *info)
 {
+    GNCImportTransInfo* trans_info;
+    Account* account;
+    Split* first_split;
+    GSList* temp_trans_list;
+
     g_assert (info);
+
+    // Set initial state of Append checkbox to same as last import for this account.
+    // Get the import account from the first split in first transaction.
+    temp_trans_list = info->temp_trans_list;
+    trans_info = temp_trans_list->data;
+    first_split = gnc_import_TransInfo_get_fsplit (trans_info);
+    account = xaccSplitGetAccount(first_split);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (info->append_text),
+                                 xaccAccountGetAppendText(account));
+
     gnc_gen_trans_list_create_matches (info);
     resolve_conflicts (info);
     gtk_widget_show_all (GTK_WIDGET(info->main_widget));
@@ -457,6 +474,9 @@ on_matcher_ok_clicked (GtkButton *button, GNCImportMainMatcher *info)
     GtkTreeModel *model;
     GtkTreeIter iter;
     GNCImportTransInfo *trans_info;
+    gboolean append_text = gtk_toggle_button_get_active ((GtkToggleButton*) info->append_text);
+    gboolean first_tran = TRUE;
+    gpointer user_data = info->user_data;
 
     g_assert (info);
 
@@ -479,6 +499,20 @@ on_matcher_ok_clicked (GtkButton *button, GNCImportMainMatcher *info)
                             DOWNLOADED_COL_DATA, &trans_info,
                             -1);
 
+        // Allow the backend to know if the Append checkbox is ticked or unticked
+        // by propagating info->append_text to every trans_info->append_text
+        gnc_import_TransInfo_set_append_text( trans_info, append_text);
+
+        // When processing the first transaction,
+        // save the state of the Append checkbox to an account kvp so the same state can be
+        //  defaulted next time this account is imported.
+        // Get the import account from the first split.
+        if (first_tran)
+        {
+            Split* first_split = gnc_import_TransInfo_get_fsplit (trans_info);
+            xaccAccountSetAppendText (xaccSplitGetAccount(first_split), append_text);
+            first_tran = FALSE;
+        }
         // Note: if there's only 1 split (unbalanced) one will be created with the unbalanced account,
         // and for that account the defer balance will not be set. So things will be slow.
 
@@ -735,6 +769,7 @@ gnc_gen_trans_assign_transfer_account (GtkTreeView *treeview,
                 old_acc  = gnc_import_TransInfo_get_destacc (trans_info);
                 if (*first)
                 {
+                    gchar *acc_full_name = gnc_account_get_full_name (*new_acc);
                     ok_pressed = FALSE;
                     *new_acc = gnc_import_select_account (info->main_widget,
                         NULL,
@@ -746,8 +781,9 @@ gnc_gen_trans_assign_transfer_account (GtkTreeView *treeview,
                         old_acc,
                         &ok_pressed);
                     *first = FALSE;
-                    DEBUG("account selected = %s",
-                            gnc_account_get_full_name (*new_acc));
+                    acc_full_name = gnc_account_get_full_name (*new_acc);
+                    DEBUG("account selected = %s", acc_full_name);
+                    g_free (acc_full_name);
                 }
                 if (ok_pressed)
                 {
@@ -805,19 +841,22 @@ gnc_gen_trans_assign_transfer_account_to_selection_cb (GtkMenuItem *menuitem,
         {
             gchar *path_str = gtk_tree_path_to_string (l->data);
             GtkTreeRowReference *ref = gtk_tree_row_reference_new (model, l->data);
+            gchar *fullname;
             DEBUG("passing first = %s", first ? "true" : "false");
             DEBUG("passing is_selection = %s", is_selection ? "true" : "false");
             DEBUG("passing path = %s", path_str);
             g_free (path_str);
             refs = g_list_prepend (refs, ref);
-            DEBUG("passing account value = %s",
-                        gnc_account_get_full_name (assigned_account));
+            fullname = gnc_account_get_full_name (assigned_account);
+            DEBUG("passing account value = %s", fullname);
+            g_free (fullname);
             gnc_gen_trans_assign_transfer_account (treeview,
                                                    &first, is_selection, l->data,
                                                    &assigned_account, info);
-            DEBUG("returned value of account = %s",
-                        gnc_account_get_full_name (assigned_account));
+            fullname = gnc_account_get_full_name (assigned_account);
+            DEBUG("returned value of account = %s", fullname);
             DEBUG("returned value of first = %s", first ? "true" : "false");
+            g_free (fullname);
             if (assigned_account == NULL)
                 break;
 
@@ -848,6 +887,7 @@ gnc_gen_trans_row_activated_cb (GtkTreeView *treeview,
 {
     Account *assigned_account;
     gboolean first, is_selection;
+    gchar *namestr;
 
     ENTER("");
     assigned_account = NULL;
@@ -859,7 +899,9 @@ gnc_gen_trans_row_activated_cb (GtkTreeView *treeview,
 
     gtk_tree_selection_select_path (gtk_tree_view_get_selection (treeview), path);
 
-    DEBUG("account returned = %s", gnc_account_get_full_name (assigned_account));
+    namestr = gnc_account_get_full_name (assigned_account);
+    DEBUG("account returned = %s", namestr);
+    g_free (namestr);
     LEAVE("");
 }
 
@@ -870,6 +912,10 @@ get_action_for_path (GtkTreePath* path, GtkTreeModel *model)
     GtkTreeIter iter;
     gtk_tree_model_get_iter (model, &iter, path);
     gtk_tree_model_get (model, &iter, DOWNLOADED_COL_DATA, &trans_info, -1);
+    if (!trans_info)
+          // selected row is a potential match  (depth 2)
+          // instead of an imported transaction (depth 1)
+          return GNCImport_INVALID_ACTION;
     return gnc_import_TransInfo_get_action (trans_info);
 }
 
@@ -972,7 +1018,6 @@ gnc_gen_trans_onButtonPressed_cb (GtkTreeView *treeview,
                 GList* selected;
                 GtkTreeModel *model;
                 selected = gtk_tree_selection_get_selected_rows (selection, &model);
-                get_action_for_path (selected->data, model);
                 if (get_action_for_path (selected->data, model) == GNCImport_ADD)
                     gnc_gen_trans_view_popup_menu (treeview, event, info);
                 g_list_free_full (selected, (GDestroyNotify)gtk_tree_path_free);
@@ -1211,7 +1256,9 @@ gnc_gen_trans_common_setup (GNCImportMainMatcher *info,
     g_signal_connect (G_OBJECT(info->show_matched_info), "toggled",
                       G_CALLBACK(show_matched_info_toggled_cb), info);
 
-    // Create the checkbox, but do not show it by default.
+    info->append_text = GTK_WIDGET(gtk_builder_get_object (builder, "append_desc_notes_button"));
+
+    // Create the checkbox, but do not show it unless there are transactions
     info->reconcile_after_close = GTK_WIDGET(gtk_builder_get_object (builder, "reconcile_after_close_button"));
 
     show_update = gnc_import_Settings_get_action_update_enabled (info->user_settings);
@@ -1347,15 +1394,19 @@ gnc_gen_trans_list_run (GNCImportMainMatcher *info)
     return result;
 }
 
-static gchar*
+static const gchar*
 get_required_color (const gchar *class_name)
 {
     GdkRGBA color;
+    static gchar *strbuf = NULL;
     GtkWidget *label = gtk_label_new ("Color");
     GtkStyleContext *context = gtk_widget_get_style_context (GTK_WIDGET(label));
     gtk_style_context_add_class (context, class_name);
     gnc_style_context_get_background_color (context, GTK_STATE_FLAG_NORMAL, &color);
-    return gdk_rgba_to_string (&color);
+    if (strbuf)
+        g_free (strbuf);
+    strbuf = gdk_rgba_to_string (&color);
+    return strbuf;
 }
 
 static void
@@ -1406,6 +1457,29 @@ update_child_row (GNCImportMatchInfo *sel_match, GtkTreeModel *model, GtkTreeIte
     g_free (text);
 }
 
+static gchar *
+get_peer_acct_names (Split *split)
+{
+    GList *names = NULL, *accounts_seen = NULL;
+    gchar *retval, *name;
+    for (GList *n = xaccTransGetSplitList (xaccSplitGetParent (split)); n; n = n->next)
+    {
+        Account *account = xaccSplitGetAccount (n->data);
+        if ((n->data == split) ||
+            (xaccAccountGetType (account) == ACCT_TYPE_TRADING) ||
+            (g_list_find (accounts_seen, account)))
+            continue;
+        name = gnc_account_get_full_name (account);
+        names = g_list_prepend (names, g_strdup_printf ("\"%s\"", name));
+        accounts_seen = g_list_prepend (accounts_seen, account);
+        g_free (name);
+    }
+    retval = gnc_g_list_stringjoin (names, ", ");
+    g_list_free_full (names, g_free);
+    g_list_free (accounts_seen);
+    return retval;
+}
+
 static void
 refresh_model_row (GNCImportMainMatcher *gui,
                    GtkTreeModel *model,
@@ -1414,8 +1488,8 @@ refresh_model_row (GNCImportMainMatcher *gui,
 {
     GtkTreeStore *store;
     GtkTreeSelection *selection;
-    gchar *tmp, *imbalance, *text, *color;
-    const gchar *ro_text;
+    gchar *tmp, *imbalance, *text;
+    const gchar *ro_text, *color = NULL;
     gchar *int_required_class, *int_prob_required_class, *int_not_required_class;
     gchar *class_extension = NULL;
     gboolean show_pixbuf = TRUE;
@@ -1470,7 +1544,7 @@ refresh_model_row (GNCImportMainMatcher *gui,
     /*Actions*/
 
     /* Action information */
-    ro_text = text = color = NULL;
+    ro_text = text = NULL;
     switch (gnc_import_TransInfo_get_action (info))
     {
     case GNCImport_ADD:
@@ -1535,15 +1609,19 @@ refresh_model_row (GNCImportMainMatcher *gui,
 
             if (sel_match)
             {
+                gchar *full_names = get_peer_acct_names (sel_match->split);
                 color = get_required_color (int_not_required_class);
                 if (gnc_import_TransInfo_get_match_selected_manually (info))
                 {
-                    ro_text = _("Reconcile (manual) match");
+                    text = g_strdup_printf (_("Reconcile (manual) match to %s"),
+                                            full_names);
                 }
                 else
                 {
-                    ro_text = _("Reconcile (auto) match");
+                    text = g_strdup_printf (_("Reconcile (auto) match to %s"),
+                                            full_names);
                 }
+                g_free (full_names);
                 update_child_row (sel_match, model, iter);
             }
             else
@@ -1561,15 +1639,19 @@ refresh_model_row (GNCImportMainMatcher *gui,
 
             if (sel_match)
             {
+                gchar *full_names = get_peer_acct_names (sel_match->split);
                 color = get_required_color (int_not_required_class);
                 if (gnc_import_TransInfo_get_match_selected_manually (info))
                 {
-                    ro_text = _("Update and reconcile (manual) match");
+                    text = g_strdup_printf (_("Update and reconcile (manual) match to %s"),
+                                            full_names);
                 }
                 else
                 {
-                    ro_text = _("Update and reconcile (auto) match");
+                    text = g_strdup_printf (_("Update and reconcile (auto) match to %s"),
+                                            full_names);
                 }
+                g_free (full_names);
                 update_child_row (sel_match, model, iter);
             }
             else
@@ -1802,6 +1884,8 @@ typedef struct _match_struct
 {
     GNCImportTransInfo* transaction_info;
     gint display_threshold;
+    gint date_threshold;
+    gint date_not_threshold;
     double fuzzy_amount;
 } match_struct;
 
@@ -1809,7 +1893,10 @@ static void
 match_helper (Split* data, match_struct* s)
 {
     split_find_match (s->transaction_info, data,
-                      s->display_threshold, s->fuzzy_amount);
+                      s->display_threshold, 
+                      s->date_threshold,
+                      s->date_not_threshold,
+                      s->fuzzy_amount);
 }
 
 /* Iterate through the imported transactions selecting matches from the
@@ -1823,6 +1910,10 @@ perform_matching (GNCImportMainMatcher *gui, GHashTable *account_hash)
     GtkTreeModel* model = gtk_tree_view_get_model (gui->view);
     gint display_threshold =
         gnc_import_Settings_get_display_threshold (gui->user_settings);
+    gint date_threshold =
+        gnc_import_Settings_get_date_threshold (gui->user_settings);
+    gint date_not_threshold =
+        gnc_import_Settings_get_date_not_threshold (gui->user_settings);
     double fuzzy_amount =
         gnc_import_Settings_get_fuzzy_amount (gui->user_settings);
 
@@ -1834,7 +1925,7 @@ perform_matching (GNCImportMainMatcher *gui, GHashTable *account_hash)
         gboolean match_selected_manually;
         GNCImportTransInfo* txn_info = imported_txn->data;
         Account *importaccount = xaccSplitGetAccount (gnc_import_TransInfo_get_fsplit (txn_info));
-        match_struct s = {txn_info, display_threshold, fuzzy_amount};
+        match_struct s = {txn_info, display_threshold, date_threshold, date_not_threshold, fuzzy_amount};
 
         g_slist_foreach (g_hash_table_lookup (account_hash, importaccount),
                          (GFunc) match_helper, &s);
@@ -1881,6 +1972,13 @@ gnc_gen_trans_list_widget (GNCImportMainMatcher *info)
     return info->main_widget;
 }
 
+GtkWidget *
+gnc_gen_trans_list_append_text_widget (GNCImportMainMatcher *info)
+{
+    g_assert (info);
+    return info->append_text;
+}
+
 gboolean
 query_tooltip_tree_view_cb (GtkWidget *widget, gint x, gint y,
                             gboolean keyboard_tip,
@@ -1897,7 +1995,10 @@ query_tooltip_tree_view_cb (GtkWidget *widget, gint x, gint y,
     gtk_tree_view_convert_widget_to_bin_window_coords (tree_view, x, y, &x, &y);
     if (keyboard_tip || !gtk_tree_view_get_path_at_pos (tree_view, x, y, &path,
                                                         &column, NULL, NULL))
+    {
+        gtk_tree_path_free (path);
         return FALSE;
+    }
 
     // Get the iter pointing to our current column
     if (gtk_tree_model_get_iter(model, &iter, path) && column)
