@@ -31,6 +31,7 @@
 #include <string.h>
 #include <sys/time.h>
 #include <math.h>
+#include <inttypes.h>
 
 #include <libofx/libofx.h>
 #include "import-account-matcher.h"
@@ -73,11 +74,15 @@ typedef struct _ofx_info
     Account *last_investment_account;
     Account *last_income_account;
     gint num_trans_processed;               // Number of transactions processed
-    struct OfxStatementData* statement;     // Statement, if any
+    GList* statement;     // Statement, if any
     gboolean run_reconcile;                 // If TRUE the reconcile window is opened after matching.
     GSList* file_list;                      // List of OFX files to import
+    GList* trans_list;                      // We store the processed ofx transactions here
+    gint response;                          // Response sent by the match gui
 } ofx_info ;
 
+static void runMatcher(ofx_info* info, char * selected_filename, gboolean go_to_next_file);
+
 /*
 int ofx_proc_status_cb(struct OfxStatusData data)
 {
@@ -210,6 +215,10 @@ static const gchar *gnc_ofx_invttype_to_str(InvTransactionType t)
         return "SPLIT (Stock or mutial fund split)";
     case OFX_TRANSFER:
         return "TRANSFER (Transfer holdings in and out of the investment account)";
+#ifdef HAVE_LIBOFX_VERSION_0_10
+    case OFX_INVBANKTRAN:
+         return "Transfer cash in and out of the investment account";
+#endif
     default:
         return "ERROR, this investment transaction type is unknown.  This is a bug in ofxdump";
     }
@@ -582,15 +591,24 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data, void *user_data)
     xaccTransSetCurrency(transaction, currency);
     if (data.amount_valid)
     {
-        if (!data.invtransactiontype_valid)
+        if (!data.invtransactiontype_valid
+#ifdef HAVE_LIBOFX_VERSION_0_10
+            || data.invtransactiontype == OFX_INVBANKTRAN)
+#else
+             )
+#endif
         {
+            double amount = data.amount;
+#ifdef HAVE_LIBOFX_VERSION_0_10
+            if (data.currency_ratio_valid && data.currency_ratio != 0)
+                amount *= data.currency_ratio;
+#endif
             /***** Process a normal transaction ******/
             DEBUG("Adding split; Ordinary banking transaction, money flows from or into the source account");
             split = xaccMallocSplit(book);
             xaccTransAppendSplit(transaction, split);
             xaccAccountInsertSplit(account, split);
-
-            gnc_amount = gnc_ofx_numeric_from_double_txn(data.amount, transaction);
+            gnc_amount = gnc_ofx_numeric_from_double_txn(amount, transaction);
             xaccSplitSetBaseValue(split, gnc_amount, xaccTransGetCurrency(transaction));
 
             /* set tran-num and/or split-action per book option */
@@ -798,11 +816,15 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data, void *user_data)
 
             if (data.invtransactiontype_valid && investment_account)
             {
+                double amount = data.amount;
+#ifdef HAVE_LIBOFX_VERSION_0_10
+                if (data.currency_ratio_valid && data.currency_ratio != 0)
+                    amount *= data.currency_ratio;
+#endif
                 if (data.invtransactiontype == OFX_REINVEST
                         || data.invtransactiontype == OFX_INCOME)
                 {
                     DEBUG("Now let's find an account for the destination split");
-
                     income_account =
                         get_associated_income_account(investment_account);
 
@@ -845,8 +867,8 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data, void *user_data)
                     split = xaccMallocSplit(book);
                     xaccTransAppendSplit(transaction, split);
                     xaccAccountInsertSplit(income_account, split);
-
-                    gnc_amount = gnc_ofx_numeric_from_double_txn (data.amount, transaction);
+                    gnc_amount = gnc_ofx_numeric_from_double_txn(amount,
+                                                                 transaction);
                     xaccSplitSetBaseValue(split, gnc_amount, xaccTransGetCurrency(transaction));
 
                     // Set split memo from ofx transaction name or memo
@@ -859,9 +881,10 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data, void *user_data)
                     split = xaccMallocSplit(book);
                     xaccTransAppendSplit(transaction, split);
                     xaccAccountInsertSplit(income_account, split);
+                    /*OFX_INCOME amounts come in as positive numbers*/
+                    gnc_amount = gnc_ofx_numeric_from_double_txn (-amount,
+                                                                  transaction);
 
-                    gnc_amount = gnc_ofx_numeric_from_double_txn (-data.amount,/*OFX_INCOME amounts come in as positive numbers*/
-                                 transaction);
                     xaccSplitSetBaseValue(split, gnc_amount, xaccTransGetCurrency(transaction));
 
                     // Set split memo from ofx transaction name or memo
@@ -890,8 +913,9 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data, void *user_data)
         /* Send transaction to importer GUI. */
         if (xaccTransCountSplits(transaction) > 0)
         {
-            DEBUG("%d splits sent to the importer gui", xaccTransCountSplits(transaction));
-            gnc_gen_trans_list_add_trans (info->gnc_ofx_importer_gui, transaction);
+            DEBUG("%d splits sent to the importer gui",
+                  xaccTransCountSplits(transaction));
+            info->trans_list = g_list_prepend (info->trans_list, transaction);
         }
         else
         {
@@ -914,8 +938,9 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data, void *user_data)
 int ofx_proc_statement_cb (struct OfxStatementData data, void * statement_user_data)
 {
     ofx_info* info = (ofx_info*) statement_user_data;
-    info->statement = g_new (struct OfxStatementData, 1);
-    *info->statement = data;
+    struct OfxStatementData *statement = g_new (struct OfxStatementData, 1);
+    *statement = data;
+    info->statement = g_list_prepend (info->statement, statement);
     return 0;
 }
 
@@ -926,6 +951,8 @@ int ofx_proc_account_cb(struct OfxAccountData data, void * account_user_data)
     gnc_commodity * default_commodity;
     GNCAccountType default_type = ACCT_TYPE_NONE;
     gchar * account_description;
+    GtkWidget * main_widget;
+    GtkWidget * parent;
     /* In order to trigger a book options display on the creation of a new book,
      * we need to detect when we are dealing with a new book. */
     gboolean new_book = gnc_is_new_book();
@@ -1006,7 +1033,17 @@ int ofx_proc_account_cb(struct OfxAccountData data, void * account_user_data)
                                                "%s \"%s\"",
                                                account_type_name,
                                                data.account_name);
-        account = gnc_import_select_account (gnc_gen_trans_list_widget(info->gnc_ofx_importer_gui),
+
+        main_widget = gnc_gen_trans_list_widget (info->gnc_ofx_importer_gui);
+
+        /* On first use, the import-main-matcher is hidden / not realized so to
+         * get a parent use the transient parent of the matcher */
+        if (gtk_widget_get_realized (main_widget))
+            parent = main_widget;
+        else
+            parent = GTK_WIDGET(gtk_window_get_transient_for (GTK_WINDOW(main_widget)));
+
+        account = gnc_import_select_account (parent,
                                              data.account_id, 1,
                                              account_description, default_commodity,
                                              default_type, NULL, NULL);
@@ -1028,6 +1065,13 @@ int ofx_proc_account_cb(struct OfxAccountData data, void * account_user_data)
 
 double ofx_get_investment_amount(const struct OfxTransactionData* data)
 {
+    double amount = data->amount;
+#ifdef HAVE_LIBOFX_VERSION_0_10
+    if (data->invtransactiontype == OFX_INVBANKTRAN)
+        return 0.0;
+    if (data->currency_ratio_valid && data->currency_ratio != 0)
+        amount *= data->currency_ratio;
+#endif
     g_assert(data);
     switch (data->invtransactiontype)
     {
@@ -1036,15 +1080,15 @@ double ofx_get_investment_amount(const struct OfxTransactionData* data)
     case OFX_BUYOPT:
     case OFX_BUYOTHER:
     case OFX_BUYSTOCK:
-        return fabs(data->amount);
+        return fabs(amount);
     case OFX_SELLDEBT:
     case OFX_SELLMF:
     case OFX_SELLOPT:
     case OFX_SELLOTHER:
     case OFX_SELLSTOCK:
-        return -1 * fabs(data->amount);
+        return -1 * fabs(amount);
     default:
-        return -1 * data->amount;
+        return -1 * amount;
     }
 }
 
@@ -1058,11 +1102,11 @@ gnc_ofx_process_next_file (GtkDialog *dialog, gpointer user_data)
 {
     ofx_info* info = (ofx_info*) user_data;
     // Free the statement (if it was allocated)
-    g_free (info->statement);
+    g_list_free_full (info->statement, g_free);
     info->statement = NULL;
 
     // Done with the previous OFX file, process the next one if any.
-    info->file_list = g_slist_delete_link(info->file_list, info->file_list);
+    info->file_list = g_slist_delete_link (info->file_list, info->file_list);
     if (info->file_list)
         gnc_file_ofx_import_process_file (info);
     else
@@ -1072,38 +1116,80 @@ gnc_ofx_process_next_file (GtkDialog *dialog, gpointer user_data)
     }
 }
 
-
-// This callback is called when the user is done matching transactions.
 static void
-gnc_ofx_match_done (GtkDialog *dialog, gint response_id, gpointer user_data)
+gnc_ofx_on_match_click (GtkDialog *dialog, gint response_id, gpointer user_data)
+{
+    // Record the response of the user. If cancel we won't go to the next file, etc.
+    ofx_info* info = (ofx_info*)user_data;
+    info->response = response_id;
+}
+
+static void
+gnc_ofx_match_done (GtkDialog *dialog, gpointer user_data)
 {
     ofx_info* info = (ofx_info*) user_data;
 
-    if (response_id == GTK_RESPONSE_OK && info->run_reconcile && info->statement)
+    /* The the user did not click OK, don't process the rest of the
+     * transaction, don't go to the next of xfile.
+     */
+    if (info->response != GTK_RESPONSE_OK)
+        return;
+
+    if (info->trans_list)
     {
+         /* Re-run the match dialog if there are transactions
+          * remaining in our list (happens if several accounts exist
+          * in the same ofx).
+          */
+        info->gnc_ofx_importer_gui = gnc_gen_trans_list_new (GTK_WIDGET (info->parent), NULL, FALSE, 42, FALSE);
+        runMatcher (info, NULL, true);
+        return;
+    }
+
+    if (info->run_reconcile && info->statement && info->statement->data)
+    {
+        struct OfxStatementData* statement = info->statement->data;
         // Open a reconcile window.
         Account* account = gnc_import_select_account (gnc_gen_trans_list_widget(info->gnc_ofx_importer_gui),
-                                                      info->statement->account_id,
+                                                      statement->account_id,
                                                       0, NULL, NULL, ACCT_TYPE_NONE, NULL, NULL);
-        if (account && info->statement->ledger_balance_valid)
+        if (account && statement->ledger_balance_valid)
         {
-            gnc_numeric value = double_to_gnc_numeric (info->statement->ledger_balance,
+            gnc_numeric value = double_to_gnc_numeric (statement->ledger_balance,
                                                        xaccAccountGetCommoditySCU (account),
                                                        GNC_HOW_RND_ROUND_HALF_UP);
 
             RecnWindow* rec_window = recnWindowWithBalance (GTK_WIDGET (info->parent), account, value,
-                                                            info->statement->ledger_balance_date);
+                                                            statement->ledger_balance_date);
 
             // Connect to destroy, at which point we'll process the next OFX file..
-            g_signal_connect (G_OBJECT(gnc_ui_reconcile_window_get_window (rec_window)), "destroy",
-                              G_CALLBACK(gnc_ofx_process_next_file), info);
+            g_signal_connect (G_OBJECT (gnc_ui_reconcile_window_get_window (rec_window)), "destroy",
+                              G_CALLBACK (gnc_ofx_match_done), info);
+            if (info->statement->next)
+                info->statement = info->statement->next;
+            else
+            {
+                g_list_free_full (g_list_first (info->statement), g_free);
+                info->statement = NULL;
+            }
+            return;
         }
     }
     else
     {
-        gtk_widget_hide (GTK_WIDGET(dialog));
-        gnc_ofx_process_next_file (dialog, info);
+        if (info->statement && info->statement->next)
+        {
+            info->statement = info->statement->next;
+            gnc_ofx_match_done (dialog, user_data);
+            return;
+        }
+        else
+        {
+            g_list_free_full (g_list_first (info->statement), g_free);
+            info->statement = NULL;
+        }
     }
+    gnc_ofx_process_next_file (NULL, info);
 }
 
 // This callback is triggered when the user checks or unchecks the reconcile after match
@@ -1114,6 +1200,110 @@ reconcile_when_close_toggled_cb (GtkToggleButton *togglebutton, ofx_info* info)
     info->run_reconcile = gtk_toggle_button_get_active (togglebutton);
 }
 
+static gchar* make_date_amount_key (time64 date, gnc_numeric amount)
+{
+    // Create a string that combines date and amount, we'll use that for our hash
+    gchar buf[64];
+    gnc_numeric _amount = gnc_numeric_reduce(amount);
+    g_snprintf (buf, sizeof(buf), "%" PRId64 "%" PRId64 "%" PRId64, _amount.num , _amount.denom, date);
+    return g_strdup (buf);
+}
+
+static void
+runMatcher (ofx_info* info, char * selected_filename, gboolean go_to_next_file)
+{
+    GtkWindow *parent = info->parent;
+    GList* trans_list_remain = NULL;
+    Account* first_account = NULL;
+
+    /* If we have multiple accounts in the ofx file, we need to
+     * avoid processing transfers between accounts together because this will
+     * create duplicate entries.
+     */
+    GHashTable* trans_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                    g_free, NULL);
+    info->num_trans_processed = 0;
+    // Add transactions, but verify that there isn't one that was already added with identical
+    // amounts and date, and a different account. To do that, create a hash table whose key is
+    // a hash of amount and date, and whose value is the account in which they appear.
+    for(GList* node = info->trans_list; node; node=node->next)
+    {
+        Transaction* trans = node->data;
+        Split* split = xaccTransGetSplit (trans, 0);
+        Account* account = xaccSplitGetAccount (split);
+        gchar *date_amount_key = make_date_amount_key (xaccTransGetDate (trans),
+                                                      gnc_numeric_abs (xaccSplitGetAmount (split)));
+        // Test if date_amount_key is already in trans_hash.
+        Account* _account = g_hash_table_lookup (trans_hash, date_amount_key);
+        if (_account && _account != account)
+        {
+            if (qof_log_check (G_LOG_DOMAIN, QOF_LOG_DEBUG))
+            {
+                // There is a transaction with identical amounts and
+                // dates, but a different account.  That's a potential
+                // transfer so process this transaction in a later call.
+                gchar *name1 = gnc_account_get_full_name (account);
+                gchar *name2 = gnc_account_get_full_name (_account);
+                gchar *amtstr = gnc_numeric_to_string (xaccSplitGetAmount (split));
+                gchar *datestr = qof_print_date (xaccTransGetDate (trans));
+                DEBUG ("Potential transfer %s %s %s %s\n", name1, name2, amtstr, datestr);
+                g_free (name1);
+                g_free (name2);
+                g_free (amtstr);
+                g_free (datestr);
+            }
+            trans_list_remain = g_list_prepend (trans_list_remain, trans);
+            g_free (date_amount_key);
+        }
+        else
+        {
+            g_hash_table_insert (trans_hash, date_amount_key, account);
+            gnc_gen_trans_list_add_trans (info->gnc_ofx_importer_gui, trans);
+            info->num_trans_processed ++;
+        }
+    }
+    g_list_free (info->trans_list);
+    g_hash_table_destroy (trans_hash);
+    info->trans_list = g_list_reverse (trans_list_remain);
+    DEBUG("%d transactions remaining to process in file %s\n", g_list_length (info->trans_list), selected_filename);
+
+    // See whether the view has anything in it and warn the user if not.
+    if (gnc_gen_trans_list_empty (info->gnc_ofx_importer_gui))
+    {
+        gnc_gen_trans_list_delete (info->gnc_ofx_importer_gui);
+        if (info->num_trans_processed)
+        {
+            gnc_info_dialog (parent, _("While importing transactions from OFX file '%s' found %d previously imported transactions, no new transactions."),
+                             selected_filename, info->num_trans_processed);
+            // This is required to ensure we don't mistakenly assume the user canceled.
+            info->response = GTK_RESPONSE_OK;
+            gnc_ofx_match_done (NULL, info);
+            return;
+        }
+    }
+    else
+    {
+        /* Show the match dialog and connect to the "destroy" signal so we can trigger a reconcile when
+         the user clicks OK when done matching transactions if required. Connecting to response isn't enough
+         because only when the matcher is destroyed do imported transactions get recorded */
+        g_signal_connect (G_OBJECT (gnc_gen_trans_list_widget (info->gnc_ofx_importer_gui)), "destroy",
+                          G_CALLBACK (gnc_ofx_match_done), info);
+        
+        // Connect to response so we know if the user pressed "cancel".
+        g_signal_connect (G_OBJECT (gnc_gen_trans_list_widget (info->gnc_ofx_importer_gui)), "response",
+                          G_CALLBACK (gnc_ofx_on_match_click), info);
+        
+        gnc_gen_trans_list_show_all (info->gnc_ofx_importer_gui);
+        
+        // Show or hide the check box for reconciling after match, depending on whether a statement was received.
+        gnc_gen_trans_list_show_reconcile_after_close_button (info->gnc_ofx_importer_gui, info->statement != NULL, info->run_reconcile);
+        
+        // Finally connect to the reconcile after match check box so we can be notified if the user wants/does not want to reconcile.
+        g_signal_connect (G_OBJECT (gnc_gen_trans_list_get_reconcile_after_close_button (info->gnc_ofx_importer_gui)), "toggled",
+                          G_CALLBACK (reconcile_when_close_toggled_cb), info);
+    }
+}
+
 // Aux function to process the OFX file in info->file_list
 static void
 gnc_file_ofx_import_process_file (ofx_info* info)
@@ -1149,38 +1339,12 @@ gnc_file_ofx_import_process_file (ofx_info* info)
     /*ofx_set_status_cb(libofx_context, ofx_proc_status_cb, 0);*/
 
     // Create the match dialog, and run the ofx file through the importer.
-    info->gnc_ofx_importer_gui = gnc_gen_trans_list_new (GTK_WIDGET(parent), NULL, TRUE, 42, FALSE);
+    info->gnc_ofx_importer_gui = gnc_gen_trans_list_new (GTK_WIDGET(parent), NULL, FALSE, 42, FALSE);
     libofx_proc_file (libofx_context, selected_filename, AUTODETECT);
-    
+
     // Free the libofx context before recursing to process the next file
     libofx_free_context(libofx_context);
-
-    // See whether the view has anything in it and warn the user if not.
-    if(gnc_gen_trans_list_empty (info->gnc_ofx_importer_gui))
-    {
-        gnc_gen_trans_list_delete (info->gnc_ofx_importer_gui);
-        if(info->num_trans_processed)
-            gnc_info_dialog (parent, _("OFX file '%s' imported, %d transactions processed, no transactions to match"),
-                                     selected_filename, info->num_trans_processed);
-        // Process the next OFX file if any.
-        gnc_ofx_process_next_file (NULL, info);
-    }
-    else
-    {
-        /* Show the match dialog and connect to the "response" signal so we can trigger a reconcile when
-           the user clicks OK when done matching transactions if required. */
-        g_signal_connect (G_OBJECT(gnc_gen_trans_list_widget (info->gnc_ofx_importer_gui)), "response",
-                          G_CALLBACK (gnc_ofx_match_done), info);
-
-        gnc_gen_trans_list_show_all (info->gnc_ofx_importer_gui);
-        
-        // Show or hide the check box for reconciling after match, depending on whether a statement was received.
-        gnc_gen_trans_list_show_reconcile_after_close_button (info->gnc_ofx_importer_gui, info->statement != NULL, info->run_reconcile);
-
-        // Finally connect to the reconcile after match check box so we can be notified if the user wants/does not want to reconcile.
-        g_signal_connect (G_OBJECT(gnc_gen_trans_list_get_reconcile_after_close_button (info->gnc_ofx_importer_gui)), "toggled",
-                          G_CALLBACK(reconcile_when_close_toggled_cb), info);
-    }
+    runMatcher(info, selected_filename,true);
     g_free(selected_filename);
 }
 
@@ -1199,7 +1363,7 @@ void gnc_file_ofx_import (GtkWindow *parent)
     GSList* iter = NULL;
     ofx_info* info = NULL;
     GtkFileFilter* filter = gtk_file_filter_new ();
-    
+
 
     ofx_PARSER_msg = false;
     ofx_DEBUG_msg = false;
@@ -1244,6 +1408,8 @@ void gnc_file_ofx_import (GtkWindow *parent)
         info->parent = parent;
         info->run_reconcile = FALSE;
         info->file_list = selected_filenames;
+        info->trans_list = NULL;
+        info->response = 0;
         // Call the aux import function.
         gnc_file_ofx_import_process_file (info);
     }
