@@ -280,8 +280,8 @@
     (let ((options (default-testing-options)))
       (set-option! options "Sorting" "Add indenting columns" #t)
       (set-option! options "General" "Table for Exporting" #f)
-      (set-option! options "General" "Common Currency" #t)
-      (set-option! options "General" "Show original currency amount" #t)
+      (set-option! options "Currency" "Common Currency" #t)
+      (set-option! options "Currency" "Show original currency amount" #t)
       (set-option! options "General" "Add options summary" 'never)
       (set-option! options "Sorting" "Primary Key" 'account-name)
       (set-option! options "Sorting" "Primary Subtotal" #t)
@@ -378,19 +378,19 @@
       (set! options (default-testing-options))
       (set-option! options "General" "Start Date" (cons 'absolute (gnc-dmy2time64 01 01 1969)))
       (set-option! options "General" "End Date" (cons 'absolute (gnc-dmy2time64 31 12 1970)))      
-      (set-option! options "Filter" "Reconcile Status" 'unreconciled)
+      (set-option! options "Filter" "Reconciled Status" 'unreconciled)
       (let ((sxml (options->sxml options "unreconciled")))
         (test-equal "filter unreconciled only, sum = -$20.00"
           '("-$20.00")
           (get-row-col sxml -1 -1)))
 
-      (set-option! options "Filter" "Reconcile Status" 'cleared)
+      (set-option! options "Filter" "Reconciled Status" 'cleared)
       (let ((sxml (options->sxml options "cleared")))
         (test-equal "filter cleared only, sum = $29.00"
           '("$29.00")
           (get-row-col sxml -1 -1)))
 
-      (set-option! options "Filter" "Reconcile Status" 'reconciled)
+      (set-option! options "Filter" "Reconciled Status" 'reconciled)
       (let ((sxml (options->sxml options "reconciled")))
         (test-equal "filter reconciled only, sum = -$8.00"
           '("-$8.00")
@@ -576,8 +576,8 @@
       (set-option! options "Accounts" "Accounts" (list usd-bank gbp-bank))
       (set-option! options "General" "Start Date" (cons 'absolute (gnc-dmy2time64 01 01 2000)))
       (set-option! options "General" "End Date" (cons 'absolute (gnc-dmy2time64 31 12 2000)))
-      (set-option! options "General" "Common Currency" #t)
-      (set-option! options "General" "Show original currency amount" #t)
+      (set-option! options "Currency" "Common Currency" #t)
+      (set-option! options "Currency" "Show original currency amount" #t)
       (let* ((sxml (options->sxml options "single column, with original currency headers")))
         (test-equal "single amount column, with original currency headers"
           (list "Date" "Num" "Description" "Memo/Notes" "Account"
@@ -650,8 +650,8 @@
       ;; test debit/credit dual columns
       (set! options (default-testing-options))
       (set-option! options "Display" "Amount" 'double)
-      (set-option! options "General" "Common Currency" #t)
-      (set-option! options "General" "Show original currency amount" #t)
+      (set-option! options "Currency" "Common Currency" #t)
+      (set-option! options "Currency" "Show original currency amount" #t)
       (set-option! options "Sorting" "Primary Key" 'date)
       (set-option! options "Sorting" "Primary Subtotal for Date Key" 'none)
       (let* ((sxml (options->sxml options "dual columns")))
@@ -733,7 +733,7 @@
 
       (set! options (default-testing-options))
       (set-option! options "Sorting" "Add indenting columns" #t)
-      (set-option! options "General" "Show original currency amount" #t)
+      (set-option! options "Currency" "Show original currency amount" #t)
       (set-option! options "Sorting" "Primary Key" 'account-name)
       (set-option! options "Sorting" "Primary Subtotal" #t)
       (set-option! options "Sorting" "Secondary Key" 'date)
@@ -750,7 +750,7 @@
       (set-option! options "Accounts" "Accounts" (gnc-account-get-descendants (gnc-account-get-root bank)))
       (set-option! options "Display" "Totals" #t)
       (set-option! options "Display" "Amount" 'double)
-      (set-option! options "General" "Show original currency amount" #t)
+      (set-option! options "Currency" "Show original currency amount" #t)
       (set-option! options "General" "Table for Exporting" #f)
       (set-option! options "Sorting" "Add indenting columns" #t)
       (set-option! options "Sorting" "Primary Key" 'account-name)
@@ -893,9 +893,9 @@
       (set-option! options "General" "End Date"
                    (cons 'absolute (gnc-dmy2time64 31 12 1970)))
       (set-option! options "Display" "Subtotal Table" #t)
-      (set-option! options "General" "Common Currency" #t)
-      (set-option! options "General" "Report Currency" foreign2)
-      (set-option! options "General" "Show original currency amount" #t)
+      (set-option! options "Currency" "Common Currency" #t)
+      (set-option! options "Currency" "Report's currency" foreign2)
+      (set-option! options "Currency" "Show original currency amount" #t)
       (set-option! options "Sorting" "Primary Key" 'account-name)
       (set-option! options "Sorting" "Primary Subtotal" #t)
       (set-option! options "Sorting" "Secondary Key" 'date)
