@@ -39,11 +39,29 @@
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define-module (gnucash report trep-engine))
+
 (use-modules (gnucash core-utils))
+(use-modules (gnucash engine))
+(use-modules (gnucash app-utils))
+(use-modules (gnucash utilities))
+(use-modules (gnucash report report-core)
+             (gnucash report report-utilities)
+             (gnucash report options-utilities)
+             (gnucash report commodity-utilities)
+             (gnucash report html-document)
+             (gnucash report html-style-info)
+             (gnucash report html-utilities)
+             (gnucash report html-table)
+             (gnucash report html-text))
 (use-modules (srfi srfi-11))
 (use-modules (srfi srfi-1))
 (use-modules (ice-9 match))
 
+(export gnc:trep-options-generator)
+(export gnc:trep-renderer)
+(export gnc:lists->csv)
+
 ;; Define the strings here to avoid typos and make changes easier.
 
 ;;Accounts
@@ -76,11 +94,16 @@
 ;;General
 (define optname-startdate (N_ "Start Date"))
 (define optname-enddate (N_ "End Date"))
+(define optname-date-source (N_ "Date Filter"))
 (define optname-table-export (N_ "Table for Exporting"))
+(define optname-infobox-display (N_ "Add options summary"))
+
+;; Currency
+(define pagename-currency (N_ "Currency"))
+(define optname-price-source (N_ "Price Source"))
 (define optname-common-currency (N_ "Common Currency"))
 (define optname-orig-currency (N_ "Show original currency amount"))
 (define optname-currency (N_ "Report's currency"))
-(define optname-infobox-display (N_ "Add options summary"))
 
 ;;Filtering
 (define pagename-filter (N_ "Filter"))
@@ -94,7 +117,7 @@
   (N_ "Transaction Filter excludes matched strings"))
 (define optname-transaction-matcher-caseinsensitive
   (N_ "Transaction Filter is case insensitive"))
-(define optname-reconcile-status (N_ "Reconcile Status"))
+(define optname-reconcile-status (N_ "Reconciled Status"))
 (define optname-void-transactions (N_ "Void Transactions"))
 (define optname-closing-transactions (N_ "Closing transactions"))
 
@@ -133,7 +156,6 @@ in the Options panel."))
   ;;  'sortkey             - sort parameter sent via qof-query
   ;;  'split-sortvalue     - function retrieves number/string for comparing splits
   ;;  'text                - text displayed in Display tab
-  ;;  'tip                 - tooltip displayed in Display tab
   ;;  'renderer-fn         - helper function to select subtotal/subheading renderer
   ;;       behaviour varies according to sortkey.
   ;;       account-types converts split->account
@@ -145,28 +167,24 @@ in the Options panel."))
               (cons 'split-sortvalue
                     (compose gnc-account-get-full-name xaccSplitGetAccount))
               (cons 'text (G_ "Account Name"))
-              (cons 'tip (G_ "Sort & subtotal by account name."))
               (cons 'renderer-fn xaccSplitGetAccount))
 
         (list 'account-code
               (cons 'sortkey (list SPLIT-ACCOUNT ACCOUNT-CODE-))
               (cons 'split-sortvalue (compose xaccAccountGetCode xaccSplitGetAccount))
               (cons 'text (G_ "Account Code"))
-              (cons 'tip (G_ "Sort & subtotal by account code."))
               (cons 'renderer-fn xaccSplitGetAccount))
 
         (list 'date
               (cons 'sortkey (list SPLIT-TRANS TRANS-DATE-POSTED))
               (cons 'split-sortvalue (compose xaccTransGetDate xaccSplitGetParent))
               (cons 'text (G_ "Date"))
-              (cons 'tip (G_ "Sort by date."))
               (cons 'renderer-fn #f))
 
         (list 'reconciled-date
               (cons 'sortkey (list SPLIT-DATE-RECONCILED))
               (cons 'split-sortvalue xaccSplitGetDateReconciled)
               (cons 'text (G_ "Reconciled Date"))
-              (cons 'tip (G_ "Sort by the Reconciled Date."))
               (cons 'renderer-fn #f))
 
         (list 'reconciled-status
@@ -175,7 +193,6 @@ in the Options panel."))
                                        (length (memv (xaccSplitGetReconcile s)
                                                      (map car reconcile-list)))))
               (cons 'text (G_ "Reconciled Status"))
-              (cons 'tip (G_ "Sort by the Reconciled Status"))
               (cons 'renderer-fn (lambda (s)
                                    (assv-ref reconcile-list
                                              (xaccSplitGetReconcile s)))))
@@ -184,28 +201,24 @@ in the Options panel."))
               (cons 'sortkey (list QUERY-DEFAULT-SORT))
               (cons 'split-sortvalue #f)
               (cons 'text (G_ "Register Order"))
-              (cons 'tip (G_ "Sort as in the register."))
               (cons 'renderer-fn #f))
 
         (list 'corresponding-acc-name
               (cons 'sortkey (list SPLIT-CORR-ACCT-NAME))
               (cons 'split-sortvalue xaccSplitGetCorrAccountFullName)
               (cons 'text (G_ "Other Account Name"))
-              (cons 'tip (G_ "Sort by account transferred from/to's name."))
               (cons 'renderer-fn (compose xaccSplitGetAccount xaccSplitGetOtherSplit)))
 
         (list 'corresponding-acc-code
               (cons 'sortkey (list SPLIT-CORR-ACCT-CODE))
               (cons 'split-sortvalue xaccSplitGetCorrAccountCode)
               (cons 'text (G_ "Other Account Code"))
-              (cons 'tip (G_ "Sort by account transferred from/to's code."))
               (cons 'renderer-fn (compose xaccSplitGetAccount xaccSplitGetOtherSplit)))
 
         (list 'amount
               (cons 'sortkey (list SPLIT-VALUE))
               (cons 'split-sortvalue xaccSplitGetValue)
               (cons 'text (G_ "Amount"))
-              (cons 'tip (G_ "Sort by amount."))
               (cons 'renderer-fn #f))
 
         (list 'description
@@ -213,7 +226,6 @@ in the Options panel."))
               (cons 'split-sortvalue (compose xaccTransGetDescription
                                               xaccSplitGetParent))
               (cons 'text (G_ "Description"))
-              (cons 'tip (G_ "Sort by description."))
               (cons 'renderer-fn (compose xaccTransGetDescription xaccSplitGetParent)))
 
         (if split-action?
@@ -221,42 +233,36 @@ in the Options panel."))
                   (cons 'sortkey (list SPLIT-ACTION))
                   (cons 'split-sortvalue xaccSplitGetAction)
                   (cons 'text (G_ "Number/Action"))
-                  (cons 'tip (G_ "Sort by check number/action."))
                   (cons 'renderer-fn #f))
 
             (list 'number
                   (cons 'sortkey (list SPLIT-TRANS TRANS-NUM))
                   (cons 'split-sortvalue (compose xaccTransGetNum xaccSplitGetParent))
                   (cons 'text (G_ "Number"))
-                  (cons 'tip (G_ "Sort by check/transaction number."))
                   (cons 'renderer-fn #f)))
 
         (list 't-number
               (cons 'sortkey (list SPLIT-TRANS TRANS-NUM))
               (cons 'split-sortvalue (compose xaccTransGetNum xaccSplitGetParent))
               (cons 'text (G_ "Transaction Number"))
-              (cons 'tip (G_ "Sort by transaction number."))
               (cons 'renderer-fn #f))
 
         (list 'memo
               (cons 'sortkey (list SPLIT-MEMO))
               (cons 'split-sortvalue xaccSplitGetMemo)
               (cons 'text (G_ "Memo"))
-              (cons 'tip (G_ "Sort by memo."))
               (cons 'renderer-fn xaccSplitGetMemo))
 
         (list 'notes
               (cons 'sortkey #f)
               (cons 'split-sortvalue (compose xaccTransGetNotes xaccSplitGetParent))
               (cons 'text (G_ "Notes"))
-              (cons 'tip (G_ "Sort by transaction notes."))
               (cons 'renderer-fn (compose xaccTransGetNotes xaccSplitGetParent)))
 
         (list 'none
               (cons 'sortkey '())
               (cons 'split-sortvalue #f)
               (cons 'text (G_ "None"))
-              (cons 'tip (G_ "Do not sort."))
               (cons 'renderer-fn #f))))
 
 (define (time64-year t64)
@@ -280,7 +286,6 @@ in the Options panel."))
   ;; Defines the different date sorting keys, as an association-list. Each entry:
   ;;  'split-sortvalue     - func retrieves number/string used for comparing splits
   ;;  'text                - text displayed in Display tab
-  ;;  'tip                 - tooltip displayed in Display tab
   ;;  'renderer-fn         - func retrieves string for subtotal/subheading renderer
   ;;         #f means the date sortkey is not grouped
   ;;         otherwise it converts split->string
@@ -289,21 +294,18 @@ in the Options panel."))
          (cons 'split-sortvalue #f)
          (cons 'date-sortvalue #f)
          (cons 'text (G_ "None"))
-         (cons 'tip (G_ "None."))
          (cons 'renderer-fn #f))
 
    (list 'daily
          (cons 'split-sortvalue (lambda (s) (time64-day (split->time64 s))))
          (cons 'date-sortvalue time64-day)
          (cons 'text (G_ "Daily"))
-         (cons 'tip (G_ "Daily."))
          (cons 'renderer-fn (lambda (s) (qof-print-date (split->time64 s)))))
 
    (list 'weekly
          (cons 'split-sortvalue (lambda (s) (time64-week (split->time64 s))))
          (cons 'date-sortvalue time64-week)
          (cons 'text (G_ "Weekly"))
-         (cons 'tip (G_ "Weekly."))
          (cons 'renderer-fn (compose gnc:date-get-week-year-string
                                      gnc-localtime
                                      split->time64)))
@@ -312,7 +314,6 @@ in the Options panel."))
          (cons 'split-sortvalue (lambda (s) (time64-month (split->time64 s))))
          (cons 'date-sortvalue time64-month)
          (cons 'text (G_ "Monthly"))
-         (cons 'tip (G_ "Monthly."))
          (cons 'renderer-fn (compose gnc:date-get-month-year-string
                                      gnc-localtime
                                      split->time64)))
@@ -321,7 +322,6 @@ in the Options panel."))
          (cons 'split-sortvalue (lambda (s) (time64-quarter (split->time64 s))))
          (cons 'date-sortvalue time64-quarter)
          (cons 'text (G_ "Quarterly"))
-         (cons 'tip (G_ "Quarterly."))
          (cons 'renderer-fn (compose gnc:date-get-quarter-year-string
                                      gnc-localtime
                                      split->time64)))
@@ -330,7 +330,6 @@ in the Options panel."))
          (cons 'split-sortvalue (lambda (s) (time64-year (split->time64 s))))
          (cons 'date-sortvalue time64-year)
          (cons 'text (G_ "Yearly"))
-         (cons 'tip (G_ "Yearly."))
          (cons 'renderer-fn (compose gnc:date-get-year-string
                                      gnc-localtime
                                      split->time64)))))
@@ -338,49 +337,40 @@ in the Options panel."))
 (define filter-list
   (list
    (list 'none
-         (cons 'text (G_ "None"))
-         (cons 'tip (G_ "Do not do any filtering.")))
+         (cons 'text (G_ "Do not do any filtering")))
 
    (list 'include
-         (cons 'text (G_ "Include Transactions to/from Filter Accounts"))
-         (cons 'tip (G_ "Include transactions to/from filter accounts only.")))
+         (cons 'text (G_ "Include Transactions to/from Filter Accounts")))
 
    (list 'exclude
-         (cons 'text (G_ "Exclude Transactions to/from Filter Accounts"))
-         (cons 'tip (G_ "Exclude transactions to/from all filter accounts.")))))
+         (cons 'text (G_ "Exclude Transactions to/from Filter Accounts")))))
 
 (define show-void-list
   (list
    (list 'non-void-only
          (cons 'how (logand CLEARED-ALL (lognot CLEARED-VOIDED)))
-         (cons 'text (G_ "Non-void only"))
-         (cons 'tip (G_ "Show only non-voided transactions.")))
+         (cons 'text (G_ "Non-void only")))
 
    (list 'void-only
          (cons 'how CLEARED-VOIDED)
-         (cons 'text (G_ "Void only"))
-         (cons 'tip (G_ "Show only voided transactions.")))
+         (cons 'text (G_ "Void only")))
 
    (list 'both
          (cons 'how CLEARED-ALL)
-         (cons 'text (G_ "Both"))
-         (cons 'tip (G_ "Show both (and include void transactions in totals).")))))
+         (cons 'text (G_ "Both (and include void transactions in totals)")))))
 
 (define show-closing-list
   (list
    (list 'exclude-closing
          (cons 'text (G_ "Exclude closing transactions"))
-         (cons 'tip (G_ "Exclude closing transactions from report."))
          (cons 'closing-match #f))
 
    (list 'include-both
          (cons 'text (G_ "Show both closing and regular transactions"))
-         (cons 'tip (G_ "Show both (and include closing transactions in totals)."))
          (cons 'closing-match 'both))
 
    (list 'closing-only
          (cons 'text (G_ "Show closing transactions only"))
-         (cons 'tip (G_ "Show only closing transactions."))
          (cons 'closing-match #t))))
 
 (define reconcile-status-list
@@ -390,53 +380,42 @@ in the Options panel."))
   ;;      (logior CLEARED-NO CLEARED-CLEARED) for unreconciled & cleared
   (list
    (list 'all
-         (cons 'text (G_ "All"))
-         (cons 'tip (G_ "Show All Transactions"))
+         (cons 'text (G_ "Show All Transactions"))
          (cons 'filter-types CLEARED-ALL))
 
    (list 'unreconciled
-         (cons 'text (G_ "Unreconciled"))
-         (cons 'tip (G_ "Unreconciled only"))
+         (cons 'text (G_ "Unreconciled only"))
          (cons 'filter-types CLEARED-NO))
 
    (list 'cleared
-         (cons 'text (G_ "Cleared"))
-         (cons 'tip (G_ "Cleared only"))
+         (cons 'text (G_ "Cleared only"))
          (cons 'filter-types CLEARED-CLEARED))
 
    (list 'reconciled
-         (cons 'text (G_ "Reconciled"))
-         (cons 'tip (G_ "Reconciled only"))
+         (cons 'text (G_ "Reconciled only"))
          (cons 'filter-types CLEARED-RECONCILED))))
 
 
 (define ascending-list
   (list
    (list 'ascend
-         (cons 'text (G_ "Ascending"))
-         (cons 'tip (G_ "Smallest to largest, earliest to latest.")))
+         (cons 'text (G_ "Ascending")))
    (list 'descend
-         (cons 'text (G_ "Descending"))
-         (cons 'tip (G_ "Largest to smallest, latest to earliest.")))))
+         (cons 'text (G_ "Descending")))))
 
 (define sign-reverse-list
   (list
    (list 'global
          (cons 'text (G_ "Use Global Preference"))
-         (cons 'tip (G_ "Use reversing option specified in global preference."))
          (cons 'acct-types #f))
    (list 'none
-         (cons 'text (G_ "None"))
-         (cons 'tip (G_ "Don't change any displayed amounts."))
+         (cons 'text (G_ "Don't change any displayed amounts"))
          (cons 'acct-types '()))
    (list 'income-expense
          (cons 'text (G_ "Income and Expense"))
-         (cons 'tip (G_ "Reverse amount display for Income and Expense Accounts."))
          (cons 'acct-types (list ACCT-TYPE-INCOME ACCT-TYPE-EXPENSE)))
    (list 'credit-accounts
          (cons 'text (G_ "Credit Accounts"))
-         (cons 'tip (G_ "Reverse amount display for Liability, Payable, Equity, \
-Credit Card, and Income accounts."))
          (cons 'acct-types (list ACCT-TYPE-LIABILITY ACCT-TYPE-PAYABLE
                                  ACCT-TYPE-EQUITY ACCT-TYPE-CREDIT
                                  ACCT-TYPE-INCOME)))))
@@ -449,8 +428,7 @@ Credit Card, and Income accounts."))
    (lambda (item)
      (vector
       (car item)
-      (keylist-get-info keylist (car item) 'text)
-      (keylist-get-info keylist (car item) 'tip)))
+      (keylist-get-info keylist (car item) 'text)))
    keylist))
 
 (define (SUBTOTAL-ENABLED? sortkey split-action?)
@@ -534,24 +512,37 @@ Credit Card, and Income accounts."))
   (gnc:options-add-date-interval!
    options gnc:pagename-general optname-startdate optname-enddate "a")
 
+  (gnc:register-trep-option
+   (gnc:make-multichoice-option
+    gnc:pagename-general optname-date-source
+    "a5" (G_ "Specify date to filter by...")
+    'posted
+    (list (vector 'posted (G_ "Date Posted"))
+          (vector 'reconciled (G_ "Reconciled Date"))
+          (vector 'entered (G_ "Date Entered")))))
+
   (gnc:register-trep-option
    (gnc:make-complex-boolean-option
-    gnc:pagename-general optname-common-currency
-    "e" (G_ "Convert all transactions into a common currency.") #f
-    #f
+    pagename-currency optname-common-currency
+    "a" (G_ "Convert all transactions into a common currency.") #f #f
     (lambda (x)
       (gnc-option-db-set-option-selectable-by-name
-       options gnc:pagename-general optname-currency x)
+       options pagename-currency optname-currency x)
       (gnc-option-db-set-option-selectable-by-name
-       options gnc:pagename-general optname-orig-currency x))))
-
-  (gnc:options-add-currency!
-   options gnc:pagename-general optname-currency "f")
+       options pagename-currency optname-orig-currency x)
+      (gnc-option-db-set-option-selectable-by-name
+       options pagename-currency optname-price-source x))))
 
   (gnc:register-trep-option
    (gnc:make-simple-boolean-option
-    gnc:pagename-general optname-orig-currency
-    "f1" (G_ "Also show original currency amounts") #f))
+    pagename-currency optname-orig-currency
+    "b" (G_ "Also show original currency amounts") #f))
+
+  (gnc:options-add-currency!
+   options pagename-currency optname-currency "c")
+
+  (gnc:options-add-price-source!
+   options pagename-currency optname-price-source "d" 'pricedb-nearest)
 
   (gnc:register-trep-option
    (gnc:make-simple-boolean-option
@@ -567,15 +558,9 @@ Credit Card, and Income accounts."))
     ;; This is an alist of conditions for displaying the infobox
     ;; 'no-match for empty-report
     ;; 'match for generated report
-    (list (vector 'no-match
-                  (G_ "If no transactions matched")
-                  (G_ "Display summary if no transactions were matched."))
-          (vector 'always
-                  (G_ "Always")
-                  (G_ "Always display summary."))
-          (vector 'never
-                  (G_ "Never")
-                  (G_ "Disable report summary.")))))
+    (list (vector 'no-match (G_ "If no transactions matched"))
+          (vector 'always (G_ "Always"))
+          (vector 'never (G_ "Never")))))
 
   ;; Filtering Options
 
@@ -611,7 +596,7 @@ blank, which will disable the filter.")
     "i2"
     (G_ "By default the transaction filter will search substring only. Set this to true to \
 enable full POSIX regular expressions capabilities. '#work|#family' will match both \
-tags within description, notes or memo. ")
+tags within description, notes or memo.")
     #f))
 
   (gnc:register-trep-option
@@ -1000,12 +985,8 @@ be excluded from periodic reporting.")
       gnc:pagename-display optname-detail-level
       "h" (G_ "Amount of detail to display per transaction.")
       disp-detail-level?
-      (list (vector 'multi-line
-                    (G_ "Multi-Line")
-                    (G_ "Display all splits in a transaction on a separate line."))
-            (vector 'single
-                    (G_ "Single")
-                    (G_ "Display one line per transaction, merging multiple splits where required.")))
+      (list (vector 'multi-line (G_ "One split per line"))
+            (vector 'single (G_ "One transaction per line")))
       #f
       (lambda (x)
         (set! disp-detail-level? x)
@@ -1017,9 +998,9 @@ be excluded from periodic reporting.")
       "m" (G_ "Display the amount?")
       amount-value
       (list
-       (vector 'none   (G_ "None") (G_ "No amount display."))
-       (vector 'single (G_ "Single") (G_ "Single Column Display."))
-       (vector 'double (G_ "Double") (G_ "Two Column Display.")))
+       (vector 'none   (G_ "Hide"))
+       (vector 'single (G_ "Single Column"))
+       (vector 'double (G_ "Two Columns")))
       #f
       (lambda (x)
         (set! amount-value x)
@@ -1051,7 +1032,7 @@ be excluded from periodic reporting.")
 ;; Here comes the big function that builds the whole table.
 
 (define (make-split-table splits options custom-calculated-cells
-                          begindate)
+                          begindate enddate c_account_1)
 
   (define (opt-val section name)
     (let ((option (gnc:lookup-option options section name)))
@@ -1080,10 +1061,10 @@ be excluded from periodic reporting.")
           (cons 'link (opt-val gnc:pagename-display (N_ "Link")))
           (cons 'amount-single (eq? amount-setting 'single))
           (cons 'amount-double (eq? amount-setting 'double))
-          (cons 'common-currency (opt-val gnc:pagename-general optname-common-currency))
+          (cons 'common-currency (opt-val pagename-currency optname-common-currency))
           (cons 'amount-original-currency
-                (and (opt-val gnc:pagename-general optname-common-currency)
-                     (opt-val gnc:pagename-general optname-orig-currency)))
+                (and (opt-val pagename-currency optname-common-currency)
+                     (opt-val pagename-currency optname-orig-currency)))
           (cons 'indenting (opt-val pagename-sorting optname-indenting))
           (cons 'subtotals-only
                 (and (opt-val pagename-sorting optname-show-subtotals-only)
@@ -1145,7 +1126,15 @@ be excluded from periodic reporting.")
           (gnc-reverse-balance acc)))
 
     (define (column-uses? param)
-      (cdr (assq param used-columns)))
+      (assq-ref used-columns param))
+
+    (define exchange-fn
+      (if (column-uses? 'common-currency)
+          (gnc:case-exchange-time-fn
+           (opt-val pagename-currency optname-price-source)
+           (opt-val pagename-currency optname-currency)
+           (gnc:accounts-get-commodities c_account_1 #f) enddate #f #f)
+          gnc:exchange-by-pricedb-nearest))
 
     (define left-columns
       (let* ((add-if (lambda (pred? . items) (if pred? items '())))
@@ -1288,7 +1277,7 @@ be excluded from periodic reporting.")
                                          (xaccSplitGetAmount s))))
            (split-currency (compose xaccAccountGetCommodity xaccSplitGetAccount))
            (row-currency (lambda (s) (if (column-uses? 'common-currency)
-                                         (opt-val gnc:pagename-general optname-currency)
+                                         (opt-val pagename-currency optname-currency)
                                          (split-currency s))))
            (friendly-debit (lambda (a) (gnc-account-get-debit-string (xaccAccountGetType a))))
            (friendly-credit (lambda (a) (gnc-account-get-credit-string (xaccAccountGetType a))))
@@ -1298,7 +1287,7 @@ be excluded from periodic reporting.")
                                 (if (column-uses? 'common-currency)
                                     (format #f " (~a)"
                                             (gnc-commodity-get-mnemonic
-                                             (opt-val gnc:pagename-general
+                                             (opt-val pagename-currency
                                                       optname-currency)))
                                     ""))))
            ;; For conversion to row-currency. Use midday as the
@@ -1306,7 +1295,7 @@ be excluded from periodic reporting.")
            ;; Otherwise it uses midnight which will likely match a
            ;; price on the previous day
            (converted-amount (lambda (s)
-                               (gnc:exchange-by-pricedb-nearest
+                               (exchange-fn
                                 (gnc:make-gnc-monetary (split-currency s)
                                                        (split-amount s))
                                 (row-currency s)
@@ -1954,7 +1943,8 @@ be excluded from periodic reporting.")
   ;; #:empty-report-message - a str or html-object displayed at the initial run
   ;; #:custom-split-filter - a split->bool function to add to the split filter
   ;; #:split->date - a split->time64 which overrides the default posted date filter
-  ;;     (see reconcile report)
+  ;;     if a derived report specifies this, the Date Filter option
+  ;;     becomes unused and should be hidden via gnc:option-make-internal!
   ;; #:split->date-include-false? - addendum to above, specifies filter behaviour if
   ;;     split->date returns #f. useful to include unreconciled splits in reconcile
   ;;     report. it can be useful for alternative date filtering, e.g. filter by
@@ -2011,6 +2001,9 @@ warning will be removed in GnuCash 5.0"))
          (enddate (gnc:time64-end-day-time
                    (gnc:date-option-absolute-time
                     (opt-val gnc:pagename-general optname-enddate))))
+         (date-source (if split->date
+                          'custom
+                          (opt-val gnc:pagename-general optname-date-source)))
          (transaction-matcher (opt-val pagename-filter optname-transaction-matcher))
          (transaction-filter-case-insensitive?
           (opt-val pagename-filter optname-transaction-matcher-caseinsensitive))
@@ -2089,7 +2082,7 @@ warning will be removed in GnuCash 5.0"))
              (value-of-X (comparator-function split-X))
              (value-of-Y (comparator-function split-Y))
              (op (if (string? value-of-X)
-                     (if ascend? string<? string>?)
+                     (if ascend? gnc:string-locale<? gnc:string-locale>?)
                      (if ascend? < >))))
         (and value-of-X (op value-of-X value-of-Y))))
 
@@ -2153,7 +2146,7 @@ warning will be removed in GnuCash 5.0"))
       (qof-query-set-book query (gnc-get-current-book))
       (xaccQueryAddAccountMatch query c_account_1 QOF-GUID-MATCH-ANY QOF-QUERY-AND)
       (xaccQueryAddClearedMatch query cleared-filter QOF-QUERY-AND)
-      (unless split->date
+      (when (eq? date-source 'posted)
         (xaccQueryAddDateMatchTT query #t begindate #t enddate QOF-QUERY-AND))
       (when (boolean? closing-match)
         (xaccQueryAddClosingTransMatch query closing-match QOF-QUERY-AND))
@@ -2182,11 +2175,19 @@ warning will be removed in GnuCash 5.0"))
         (filter
          (lambda (split)
            (let* ((trans (xaccSplitGetParent split)))
-             (and (or (not split->date)
-                      (let ((date (split->date split)))
+             (and (case date-source
+                    ((posted) #t)
+                    ((reconciled)
+                     (if (char=? (xaccSplitGetReconcile split) #\y)
+                         (<= begindate (xaccSplitGetDateReconciled split) enddate)
+                         #t))
+                    ((entered) (<= begindate (xaccTransRetDateEntered trans) enddate))
+                    ((custom)
+                     (let ((date (split->date split)))
                         (if date
                             (<= begindate date enddate)
                             split->date-include-false?)))
+                    (else (gnc:warn "invalid date-source" date-source) #t))
                   (case filter-mode
                     ((none) #t)
                     ((include) (is-filter-member split c_account_2))
@@ -2223,7 +2224,7 @@ warning will be removed in GnuCash 5.0"))
        (else
         (let-values (((table grid csvlist)
                       (make-split-table splits options custom-calculated-cells
-                                        begindate)))
+                                        begindate enddate c_account_1)))
 
           (gnc:html-document-set-title! document report-title)
 
@@ -2245,7 +2246,7 @@ warning will be removed in GnuCash 5.0"))
           (when subtotal-table?
             (let* ((generic<?
                     (lambda (a b)
-                      (cond ((string? (car a)) (string<? (car a) (car b)))
+                      (cond ((string? (car a)) (gnc:string-locale<? (car a) (car b)))
                             ((number? (car a)) (< (car a) (car b)))
                             (else (gnc:error "unknown sortvalue")))))
                    (list-of-rows
