@@ -16,8 +16,140 @@
 ;; Free Software Foundation           Voice:  +1-617-542-5942
 ;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
 ;; Boston, MA  02110-1301,  USA       gnu@gnu.org
-(use-modules (ice-9 regex))
+
+(define-module (gnucash app-utils options))
+
+(eval-when (compile load eval expand)
+  (load-extension "libgnc-app-utils" "scm_init_sw_app_utils_module"))
+
 (use-modules (gnucash core-utils))
+(use-modules (gnucash engine))
+(use-modules (sw_app_utils))
+(use-modules (gnucash app-utils date-utilities))
+(use-modules (gnucash utilities))
+(use-modules (srfi srfi-1))
+(use-modules (ice-9 regex))
+
+(export gnc:color->html)
+(export gnc:color-option->hex-string)
+(export gnc:color-option->html)
+(export gnc:currency-accounting-option-get-curr-doc-string)
+(export gnc:currency-accounting-option-get-default-curr)
+(export gnc:currency-accounting-option-get-default-policy)
+(export gnc:currency-accounting-option-get-gain-loss-account-doc-string)
+(export gnc:currency-accounting-option-get-policy-doc-string)
+(export gnc:currency-accounting-option-selected-currency)
+(export gnc:currency-accounting-option-selected-gain-loss-account)
+(export gnc:currency-accounting-option-selected-method)
+(export gnc:currency-accounting-option-selected-policy)
+(export gnc:date-option-absolute-time)
+(export gnc:date-option-get-subtype)
+(export gnc:date-option-relative-time)
+(export gnc:date-option-show-time?)
+(export gnc:date-option-value-type)
+(export gnc:dateformat-get-format)
+(export gnc:generate-restore-forms)
+(export gnc:get-rd-option-data-rd-list)
+(export gnc:get-rd-option-data-show-time)
+(export gnc:get-rd-option-data-subtype)
+(export gnc:lookup-option)
+(export gnc:make-account-list-limited-option)
+(export gnc:make-account-list-option)
+(export gnc:make-account-sel-limited-option)
+(export gnc:make-account-sel-option)
+(export gnc:make-budget-option)
+(export gnc:make-color-option)
+(export gnc:make-commodity-option)
+(export gnc:make-complex-boolean-option)
+(export gnc:make-currency-option)
+(export gnc:make-date-option)
+(export gnc:make-dateformat-option)
+(export gnc:make-font-option)
+(export gnc:make-internal-option)
+(export gnc:make-list-option)
+(export gnc:make-multichoice-callback-option)
+(export gnc:make-multichoice-option)
+(export gnc:make-number-plot-size-option)
+(export gnc:make-number-range-option)
+(export gnc:make-option)
+(export gnc:make-pixmap-option)
+(export gnc:make-query-option)
+(export gnc:make-radiobutton-callback-option)
+(export gnc:make-radiobutton-option)
+(export gnc:make-simple-boolean-option)
+(export gnc:make-string-option)
+(export gnc:make-text-option)
+(export gnc:multichoice-list-lookup)
+(export gnc:new-options)
+(export gnc:option-data)
+(export gnc:option-data-fns)
+(export gnc:option-default-getter)
+(export gnc:option-default-value)
+(export gnc:option-documentation)
+(export gnc:option-generate-restore-form)
+(export gnc:option-get-value)
+(export gnc:option-getter)
+(export gnc:option-index-get-name)
+(export gnc:option-index-get-value)
+(export gnc:option-kvp->scm)
+(export gnc:option-make-internal!)
+(export gnc:option-name)
+(export gnc:option-number-of-indices)
+(export gnc:option-scm->kvp)
+(export gnc:option-section)
+(export gnc:option-set-changed-callback)
+(export gnc:option-set-default-value)
+(export gnc:option-set-value)
+(export gnc:option-setter)
+(export gnc:option-sort-tag)
+(export gnc:option-strings-getter)
+(export gnc:option-type)
+(export gnc:option-value)
+(export gnc:option-value-get-index)
+(export gnc:option-value-validator)
+(export gnc:option-widget-changed-proc)
+(export gnc:options-clear-changes)
+(export gnc:options-copy-values)
+(export gnc:options-for-each)
+(export gnc:options-for-each-general)
+(export gnc:options-get-default-section)
+(export gnc:options-kvp->scm)
+(export gnc:options-make-date-interval!)
+(export gnc:options-make-end-date!)
+(export gnc:options-register-c-callback)
+(export gnc:options-register-callback)
+(export gnc:options-run-callbacks)
+(export gnc:options-scm->kvp)
+(export gnc:options-set-default-section)
+(export gnc:options-touch)
+(export gnc:options-unregister-callback-id)
+(export gnc:plot-size-option-value)
+(export gnc:plot-size-option-value-type)
+(export gnc:register-option)
+(export gnc:restore-form-generator)
+(export gnc:send-options)
+(export gnc:set-option-kvp->scm)
+(export gnc:set-option-scm->kvp)
+(export gnc:unregister-option)
+(export gnc:value->string)
+
+(export gnc:*option-name-trading-accounts*)
+(export gnc:*option-name-book-currency*)
+(export gnc:*option-section-accounts*)
+(export gnc:*option-name-default-gains-policy*)
+(export gnc:*option-name-default-gain-loss-account*)
+
+(define gnc:*option-section-accounts* OPTION-SECTION-ACCOUNTS)
+(define gnc:*option-name-trading-accounts* OPTION-NAME-TRADING-ACCOUNTS)
+(define gnc:*option-name-currency-accounting* OPTION-NAME-CURRENCY-ACCOUNTING)
+(define gnc:*option-name-book-currency* OPTION-NAME-BOOK-CURRENCY)
+(define gnc:*option-name-default-gains-policy* OPTION-NAME-DEFAULT-GAINS-POLICY)
+(define gnc:*option-name-default-gain-loss-account*
+  OPTION-NAME-DEFAULT-GAINS-LOSS-ACCT-GUID)
+
+(define (gnc:option-get-value book category key)
+  (define acc (if (pair? key) cons list))
+  (qof-book-get-option book (acc category key)))
 
 (define (rpterror-earlier type newoption fallback)
   ;; Translators: the 3 ~a below refer to (1) option type (2) unknown
@@ -71,8 +203,7 @@ the option '~a'."))
          ;; Function 3: taking one argument, a non-negative integer,
          ;; that returns the string matching the nth choice
          ;;
-         ;; Function 4: takes one argument and returns the description
-         ;; containing the nth choice
+         ;; Function 4: #f, this was the individual tool tip and not used now
          ;;
          ;; Function 5: giving a possible value and returning the index
          ;; if an option doesn't use these,  this should just be a #f
@@ -165,11 +296,6 @@ the option '~a'."))
          (name-fn (vector-ref option-data-fns 2)))
     (name-fn index)))
 
-(define (gnc:option-index-get-description option index)
-  (let* ((option-data-fns (gnc:option-data-fns option))
-         (name-fn (vector-ref option-data-fns 3)))
-    (name-fn index)))
-
 (define (gnc:option-index-get-value option index)
   (let* ((option-data-fns (gnc:option-data-fns option))
          (name-fn (vector-ref option-data-fns 1)))
@@ -340,9 +466,14 @@ the option '~a'."))
          sort-tag
          documentation-string)
 
+  (define (convert-to-guid item)
+    (if (string? item) item (gncBudgetGetGUID item)))
+
+  (define (convert-to-budget item)
+    (if (string? item) (gnc-budget-lookup item (gnc-get-current-book)) item))
+
   (let* ((initial-budget (gnc-budget-get-default (gnc-get-current-book)))
-	 (selection-budget initial-budget)
-         )
+	 (selection-budget (convert-to-guid initial-budget)))
 
     (gnc:make-option
      section 
@@ -352,12 +483,12 @@ the option '~a'."))
      documentation-string
 
      ;; getter -- Return a budget pointer
-     (lambda () 
-       selection-budget)
+     (lambda ()
+       (convert-to-budget selection-budget))
 
      ;; setter -- takes a budget
      (lambda (x)
-       (set! selection-budget x))
+       (set! selection-budget (convert-to-guid x)))
 
      ;; default-getter
      ;; Default now is #f so saving is independent of book-level default
@@ -377,21 +508,20 @@ the option '~a'."))
 	"(lambda (option) "
 	"(if option ((gnc:option-setter option) "
 	"(gnc-budget-lookup "
-	(gnc:value->string (gncBudgetGetGUID selection-budget))
+	(gnc:value->string selection-budget)
 	" (gnc-get-current-book)))))"))
 
      ;; scm->kvp -- commit the change
      ;; b -- book;  p -- key-path
      (lambda (b p) 
-       (qof-book-set-option 
-	b (gncBudgetGetGUID selection-budget) p))
+       (qof-book-set-option b selection-budget p))
 
      ;; kvp->scm -- get the stored value
      (lambda (b p)
        (let ((v (qof-book-get-option b p)))
          (if (and v (string? v))
-	     (begin 
-	       (set! selection-budget (gnc-budget-lookup v (gnc-get-current-book)))))))
+	     (set! selection-budget (convert-to-guid
+                                     (gnc-budget-lookup v (gnc-get-current-book)))))))
 
      ;; value-validator -- returns (#t value) or (#f "failure message")
      ;; As no user-generated input, this legacy hard-wire is probably ok
@@ -593,10 +723,14 @@ the option '~a'."))
         (begin
           (rpterror-earlier "date" item (car full-list))
           0)))
-  (let* ((value (default-getter))
+  (if show-time
+      (issue-deprecation-warning
+       (format #f "Date options with time of day values are deprecated and will be removed in GnuCash 5.")))
+
+ (let* ((value (default-getter))
          (value->string (lambda ()
                           (string-append "'" (gnc:value->string value)))))
-    (gnc:make-option
+     (gnc:make-option
      section name sort-tag 'date documentation-string
      (lambda () value)
      (lambda (date)
@@ -867,6 +1001,14 @@ the option '~a'."))
         (rpterror-earlier "multichoice" item (car full-lst))
         0)))
 
+(define (check-ok-values ok-values fn)
+  (for-each
+   (lambda (ok-value)
+     (when (> (vector-length ok-value) 2)
+       (issue-deprecation-warning
+        (format #f "~a: the tooltip in ~a is not supported anymore. Please remove." fn ok-value))))
+   ok-values))
+
 ;; multichoice options use the option-data as a list of vectors.
 ;; Each vector contains a permissible value (scheme symbol), a
 ;; name, and a description string.
@@ -914,8 +1056,9 @@ the option '~a'."))
     (if (null? p-vals)
         '()
         (cons (vector-ref (car p-vals) 1)
-              (cons (vector-ref (car p-vals) 2)
-                    (multichoice-strings (cdr p-vals))))))
+              (multichoice-strings (cdr p-vals)))))
+
+  (check-ok-values ok-values "gnc:make-multichoice-[callback-]option")
 
   (let* ((value default-value)
          (value->string (lambda ()
@@ -924,12 +1067,18 @@ the option '~a'."))
      section name sort-tag 'multichoice documentation-string
      (lambda () value)
      (lambda (x)
-       (if (multichoice-legal x ok-values)
-           (begin
-             (set! value x)
-             (if (procedure? setter-function-called-cb)
-                 (setter-function-called-cb x)))
-           (rpterror-earlier "multichoice" x default-value)))
+       (cond
+        ((and (equal? section "Display")
+              (equal? name "Parent account subtotals")
+              (equal? x 'canonically-tabbed))
+         (gnc:warn "canonically-tabbed obsolete. switching to 't")
+         (set! value 't))
+        ((not (multichoice-legal x ok-values))
+         (rpterror-earlier "multichoice" x default-value))
+        (else
+         (set! value x)
+         (if (procedure? setter-function-called-cb)
+             (setter-function-called-cb x)))))
      (lambda () default-value)
      (gnc:restore-form-generator value->string)
      (lambda (b p) (qof-book-set-option b (symbol->string value) p))
@@ -945,7 +1094,7 @@ the option '~a'."))
      (vector (lambda () (length ok-values))
              (lambda (x) (vector-ref (list-ref ok-values x) 0))
              (lambda (x) (vector-ref (list-ref ok-values x) 1))
-             (lambda (x) (vector-ref (list-ref ok-values x) 2))
+             #f                         ;old tooltip
              (lambda (x)
                (gnc:multichoice-list-lookup ok-values x)))
      (lambda () (multichoice-strings ok-values)) 
@@ -1000,8 +1149,7 @@ the option '~a'."))
     (if (null? p-vals)
         '()
         (cons (vector-ref (car p-vals) 1)
-              (cons (vector-ref (car p-vals) 2)
-                    (radiobutton-strings (cdr p-vals))))))
+              (radiobutton-strings (cdr p-vals)))))
 
   (let* ((value default-value)
          (value->string (lambda ()
@@ -1031,7 +1179,7 @@ the option '~a'."))
      (vector (lambda () (length ok-values))
              (lambda (x) (vector-ref (list-ref ok-values x) 0))
              (lambda (x) (vector-ref (list-ref ok-values x) 1))
-             (lambda (x) (vector-ref (list-ref ok-values x) 2))
+             #f                         ;old tooltip
              (lambda (x)
                (gnc:multichoice-list-lookup ok-values x)))
      (lambda () (radiobutton-strings ok-values)) 
@@ -1065,8 +1213,9 @@ the option '~a'."))
     (if (null? p-vals)
         '()
         (cons (vector-ref (car p-vals) 1)
-              (cons (vector-ref (car p-vals) 2)
-                    (list-strings (cdr p-vals))))))
+              (list-strings (cdr p-vals)))))
+
+  (check-ok-values ok-values "gnc:make-list-option")
 
   (let* ((value default-value)
          (value->string (lambda ()
@@ -1103,7 +1252,7 @@ the option '~a'."))
      (vector (lambda () (length ok-values))
              (lambda (x) (vector-ref (list-ref ok-values x) 0))
              (lambda (x) (vector-ref (list-ref ok-values x) 1))
-             (lambda (x) (vector-ref (ref ok-values x) 2))
+             #f                         ;old tooltip
              (lambda (x) (gnc:multichoice-list-lookup ok-values x)))
      (lambda () (list-strings ok-values)) #f)))
 
@@ -1681,10 +1830,26 @@ the option '~a'."))
       ("Void Transactions" "Filter" "Void Transactions")
       ("Account Substring" "Filter" "Account Name Filter")
       ("Enable links" #f "Enable Links")
+      ;; trep-engine: moved currency options to own tab
+      ("Common Currency" "Currency" "Common Currency")
+      ("Show original currency amount" "Currency" "Show original currency amount")
+      ("Report's currency" "Currency" "Report's currency")
+      ("Reconcile Status" #f "Reconciled Status")
+      ;; new-owner-report.scm, renamed Oct 2020 to differentiate with
+      ;; Document Links:
+      ("Links" #f "Transaction Links")
       ;; invoice.scm, renamed November 2018
       ("Individual Taxes" #f "Use Detailed Tax Summary")
+      ;; renamed in several reports, April 2021
+      ("Show Accounts until level" #f "Levels of Subaccounts")
+      ;; receipt.scm, renamed July 2021
+      ("Invoice number" #f "Invoice Number")
+      ;; receipt.scm and taxinvoice.scm, renamed July 2021
+      ("Report title" #f "Report Title")
+      ("Extra notes" #f "Extra Notes")
       ;; income-gst-statement.scm
       ("default format" #f "Default Format")
+      ("Report format" #f "Report Format")
       ))
 
   (define (lookup-option section name)
@@ -1705,7 +1870,7 @@ the option '~a'."))
                  (and name-match
                       (let ((new-section (car name-match))
                             (new-name (cadr name-match)))
-                        (gnc:debug
+                        (gnc:warn
                          (format #f "option ~a/~a has been renamed to ~a/~a\n"
                                  section name new-section new-name))
                         (cond
@@ -1937,12 +2102,6 @@ the option '~a'."))
 (define (gnc:generate-restore-forms options options-string)
   ((options 'generate-restore-forms) options-string))
 
-(define (gnc:options-fancy-date book)
-  (let ((date-format (gnc:fancy-date-info book gnc:*fancy-date-format*)))
-    (if (boolean? date-format) ;; date-format does not exist
-        (qof-date-format-get-string (qof-date-format-get))
-       date-format)))
-
 (define (gnc:options-scm->kvp options book clear-option?)
   (if clear-option?
       (qof-book-options-delete book '()))
