@@ -49,10 +49,19 @@
 typedef struct _GncCommodityClass gnc_commodityClass;
 typedef struct _GncCommodityNamespaceClass gnc_commodity_namespaceClass;
 
+#ifdef __cplusplus
+extern "C++" {
 #include <glib.h>
 #include <glib/gi18n.h>
+}
+#endif
+
 #include "gnc-engine.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* --- type macros --- */
 #define GNC_TYPE_COMMODITY            (gnc_commodity_get_type ())
 #define GNC_COMMODITY(o)              \
@@ -111,6 +120,11 @@ GType gnc_commodity_namespace_get_type(void);
    when presenting the currency related namespace to the user */
 #define GNC_COMMODITY_NS_ISO_GUI N_("Currencies")
 
+/** Max fraction is 10^9 because 10^10 would require changing it to an
+ * int64_t.
+ */
+#define GNC_COMMODITY_MAX_FRACTION 1000000000
+
 typedef GList CommodityList;
 
 /** @name Commodity Quote Source functions
@@ -139,13 +153,21 @@ typedef enum
 } QuoteSourceType;
 
 /** This function indicates whether or not the Finance::Quote module
- *  is installed on a users computer.  This includes any other related
+ *  is installed on a user's computer.  This includes any other related
  *  modules that gnucash need to process F::Q information.
  *
  *  @return TRUE is F::Q is installed properly.
  */
 gboolean gnc_quote_source_fq_installed (void);
 
+/** This function returns the version of the Finance::Quote module
+ *  installed on a user's computer. If no proper installation is found
+ *  it will return NULL.
+ *
+ *  @return a version string or NULL
+ */
+const char* gnc_quote_source_fq_version (void);
+
 /** Update gnucash internal tables based on what Finance::Quote
  *  sources are installed.  Sources that have been explicitly coded
  *  into gnucash are marked sensitive/insensitive based upon whether
@@ -155,7 +177,8 @@ gboolean gnc_quote_source_fq_installed (void);
  *  @param sources_list A list of strings containing the source names
  *  as they are known to F::Q.
  */
-void gnc_quote_source_set_fq_installed (const GList *sources_list);
+void gnc_quote_source_set_fq_installed (const char* version_string,
+                                        const GList *sources_list);
 
 /** Return the number of entries for a given type of quote source.
  *
@@ -340,7 +363,7 @@ const char * gnc_commodity_get_mnemonic(const gnc_commodity * cm);
  */
 const char * gnc_commodity_get_namespace(const gnc_commodity * cm);
 
-/** Retrieve the namespace data strucure for the specified commodity.
+/** Retrieve the namespace data structure for the specified commodity.
  *  This will be a pointer to another data structure.
  *
  *  @param cm A pointer to a commodity data structure.
@@ -786,18 +809,18 @@ gboolean gnc_commodity_table_add_default_data(gnc_commodity_table *table, QofBoo
  @{
 */
 
-/** Return the textual name of a namespace data strucure.
+/** Return the textual name of a namespace data structure.
  *
- *  @param ns A pointer to the namespace data strucure.
+ *  @param ns A pointer to the namespace data structure.
  *
  *  @return A pointer to the name of the namespace.  This string is
  *  owned by the engine and should not be freed by the caller. */
 const char * gnc_commodity_namespace_get_name (const gnc_commodity_namespace *ns) ;
 
-/** Return the textual name of a namespace data strucure in a form suitable to
+/** Return the textual name of a namespace data structure in a form suitable to
  * present to the user.
  *
- *  @param ns A pointer to the namespace data strucure.
+ *  @param ns A pointer to the namespace data structure.
  *
  *  @return A pointer to the gui friendly name of the namespace.  This string is
  *  owned by the engine and should not be freed by the caller.
@@ -953,8 +976,8 @@ gboolean gnc_commodity_table_foreach_commodity(const gnc_commodity_table * table
  @{
 */
 
-/** You proably shouldn't be using gnc_commodity_table_new() directly,
- * its for internal use only. You should probably be using
+/** You probably shouldn't be using gnc_commodity_table_new() directly,
+ * it's for internal use only. You should probably be using
  * gnc_commodity_table_get_table()
  */
 gnc_commodity_table * gnc_commodity_table_new(void);
@@ -1056,6 +1079,10 @@ void gnc_monetary_list_free(MonetaryList *list);
 
 /** @} */
 
+#ifdef __cplusplus
+} /* extern "C" */
+#endif
+
 #endif /* GNC_COMMODITY_H */
 /** @} */
 /** @} */
