@@ -558,12 +558,8 @@ gnc_price_set_typestr(GNCPrice *p, const char* type)
     if (!p) return;
     if (g_strcmp0(p->type, type) != 0)
     {
-        gchar *tmp;
-
         gnc_price_begin_edit (p);
-        tmp = CACHE_INSERT((gpointer) type);
-        if (p->type) CACHE_REMOVE(p->type);
-        p->type = tmp;
+        CACHE_REPLACE(p->type, type);
         gnc_price_set_dirty(p);
         gnc_price_commit_edit (p);
     }
@@ -800,26 +796,30 @@ gnc_price_list_destroy(PriceList *prices)
 gboolean
 gnc_price_list_equal(PriceList *prices1, PriceList *prices2)
 {
-    GList *n1, *n2;
+    GList *n1 = prices1;
+    GList *n2 = prices2;
 
     if (prices1 == prices2) return TRUE;
 
-    if (g_list_length (prices1) < g_list_length (prices2))
+    while (n1 || n2)
     {
-        PWARN ("prices2 has extra prices");
-        return FALSE;
-    }
-
-    if (g_list_length (prices1) > g_list_length (prices2))
-    {
-        PWARN ("prices1 has extra prices");
-        return FALSE;
-    }
-
-    for (n1 = prices1, n2 = prices2; n1 ; n1 = n1->next, n2 = n2->next)
+        if (!n1)
+        {
+            PINFO ("prices2 has extra prices");
+            return FALSE;
+        }
+        if (!n2)
+        {
+            PINFO ("prices1 has extra prices");
+            return FALSE;
+        }
         if (!gnc_price_equal (n1->data, n2->data))
             return FALSE;
 
+        n1 = n1->next;
+        n2 = n2->next;
+    };
+
     return TRUE;
 }
 
@@ -1075,7 +1075,6 @@ add_price(GNCPriceDB *db, GNCPrice *p)
     gnc_commodity *commodity;
     gnc_commodity *currency;
     GHashTable *currency_hash;
-    GNCPrice *old_price;
 
     if (!db || !p) return FALSE;
     ENTER ("db=%p, pr=%p dirty=%d destroying=%d",
@@ -1112,17 +1111,20 @@ add_price(GNCPriceDB *db, GNCPrice *p)
  * add this one. If this price is of equal or better precedence than the old
  * one, copy this one over the old one.
  */
-    old_price = gnc_pricedb_lookup_day_t64 (db, p->commodity, p->currency,
-                                        p->tmspec);
-    if (!db->bulk_update && old_price != NULL)
+    if (!db->bulk_update)
     {
-        if (p->source > old_price->source)
+        GNCPrice *old_price = gnc_pricedb_lookup_day_t64(db, p->commodity,
+                                                         p->currency, p->tmspec);
+        if (old_price != NULL)
         {
-            gnc_price_unref(p);
-            LEAVE ("Better price already in DB.");
-            return FALSE;
+            if (p->source > old_price->source)
+            {
+                gnc_price_unref(p);
+                LEAVE ("Better price already in DB.");
+                return FALSE;
+            }
+            gnc_pricedb_remove_price(db, old_price);
         }
-        gnc_pricedb_remove_price(db, old_price);
     }
 
     currency_hash = g_hash_table_lookup(db->commodity_hash, commodity);
@@ -1878,16 +1880,6 @@ price_list_scan_any_currency(GList *price_list, gpointer data)
     return TRUE;
 }
 
-static gboolean
-is_in_list (GList *list, const gnc_commodity *c)
-{
-    GList *node;
-    for (node = list; node != NULL; node = g_list_next(node))
-        if ((gnc_commodity*)node->data == c)
-            return TRUE;
-    return FALSE;
-}
-
 /* This operates on the principal that the prices are sorted by date and that we
  * want only the first one before the specified time containing both the target
  * and some other commodity. */
@@ -1901,17 +1893,17 @@ latest_before (PriceList *prices, const gnc_commodity* target, time64 t)
         gnc_commodity *com = gnc_price_get_commodity(price);
         gnc_commodity *cur = gnc_price_get_currency(price);
         time64 price_t = gnc_price_get_time64(price);
+
         if (t < price_t ||
-            (com == target && is_in_list(found_coms, cur)) ||
-            (cur == target && is_in_list(found_coms, com)))
+            (com == target && g_list_find (found_coms, cur)) ||
+            (cur == target && g_list_find (found_coms, com)))
             continue;
-        else
-        {
-            gnc_price_ref(price);
-            retval = g_list_prepend(retval, price);
-            found_coms = g_list_prepend(found_coms, com == target ? cur : com);
-        }
+
+        gnc_price_ref (price);
+        retval = g_list_prepend (retval, price);
+        found_coms = g_list_prepend (found_coms, com == target ? cur : com);
     }
+    g_list_free (found_coms);
     return g_list_reverse(retval);
 }
 
@@ -2024,8 +2016,8 @@ PriceList *
 gnc_pricedb_lookup_latest_any_currency(GNCPriceDB *db,
                                        const gnc_commodity *commodity)
 {
-    return gnc_pricedb_lookup_latest_before_any_currency_t64(db, commodity,
-                                                         gnc_time(NULL));
+    return gnc_pricedb_lookup_nearest_before_any_currency_t64(db, commodity,
+                                                              gnc_time(NULL));
 }
 
 PriceList *
@@ -2049,9 +2041,9 @@ gnc_pricedb_lookup_nearest_in_time_any_currency_t64(GNCPriceDB *db,
 }
 
 PriceList *
-gnc_pricedb_lookup_latest_before_any_currency_t64(GNCPriceDB *db,
-                                                  const gnc_commodity *commodity,
-                                                  time64 t)
+gnc_pricedb_lookup_nearest_before_any_currency_t64(GNCPriceDB *db,
+                                                   const gnc_commodity *commodity,
+                                                   time64 t)
 {
     GList *prices = NULL, *result;
     UsesCommodity helper = {&prices, commodity, t};
@@ -2405,10 +2397,10 @@ gnc_pricedb_lookup_nearest_in_time64(GNCPriceDB *db,
 
 
 GNCPrice *
-gnc_pricedb_lookup_latest_before_t64 (GNCPriceDB *db,
-                                      gnc_commodity *c,
-                                      gnc_commodity *currency,
-                                      time64 t)
+gnc_pricedb_lookup_nearest_before_t64 (GNCPriceDB *db,
+                                       const gnc_commodity *c,
+                                       const gnc_commodity *currency,
+                                       time64 t)
 {
     GList *price_list;
     GNCPrice *current_price = NULL;
@@ -2449,7 +2441,7 @@ extract_common_prices (PriceList *from_prices, PriceList *to_prices,
 {
     PriceTuple retval = {NULL, NULL};
     GList *from_node = NULL, *to_node = NULL;
-    GNCPrice *from_price, *to_price;
+    GNCPrice *from_price = NULL, *to_price = NULL;
 
     for (from_node = from_prices; from_node != NULL;
          from_node = g_list_next(from_node))
@@ -2516,7 +2508,7 @@ convert_price (const gnc_commodity *from, const gnc_commodity *to, PriceTuple tu
 
 static gnc_numeric
 indirect_price_conversion (GNCPriceDB *db, const gnc_commodity *from,
-                           const gnc_commodity *to, time64 t)
+                           const gnc_commodity *to, time64 t, gboolean before_date)
 {
     GList *from_prices = NULL, *to_prices = NULL;
     PriceTuple tuple;
@@ -2531,6 +2523,12 @@ indirect_price_conversion (GNCPriceDB *db, const gnc_commodity *from,
         if (from_prices)
             to_prices = gnc_pricedb_lookup_latest_any_currency(db, to);
     }
+    else if (before_date)
+    {
+        from_prices = gnc_pricedb_lookup_nearest_before_any_currency_t64 (db, from, t);
+        if (from_prices)
+            to_prices = gnc_pricedb_lookup_nearest_before_any_currency_t64 (db, to, t);
+    }
     else
     {
         from_prices = gnc_pricedb_lookup_nearest_in_time_any_currency_t64 (db, from, t);
@@ -2538,7 +2536,11 @@ indirect_price_conversion (GNCPriceDB *db, const gnc_commodity *from,
             to_prices = gnc_pricedb_lookup_nearest_in_time_any_currency_t64 (db, to, t);
     }
     if (!from_prices || !to_prices)
+    {
+        gnc_price_list_destroy (from_prices);
+        gnc_price_list_destroy (to_prices);
         return zero;
+    }
     tuple = extract_common_prices (from_prices, to_prices, from, to);
     gnc_price_list_destroy (from_prices);
     gnc_price_list_destroy (to_prices);
@@ -2550,7 +2552,7 @@ indirect_price_conversion (GNCPriceDB *db, const gnc_commodity *from,
 
 static gnc_numeric
 direct_price_conversion (GNCPriceDB *db, const gnc_commodity *from,
-                         const gnc_commodity *to, time64 t)
+                         const gnc_commodity *to, time64 t, gboolean before_date)
 {
     GNCPrice *price;
     gnc_numeric retval = gnc_numeric_zero();
@@ -2559,6 +2561,8 @@ direct_price_conversion (GNCPriceDB *db, const gnc_commodity *from,
 
     if (t == INT64_MAX)
         price = gnc_pricedb_lookup_latest(db, from, to);
+    else if (before_date)
+        price = gnc_pricedb_lookup_nearest_before_t64(db, from, to, t);
     else
         price = gnc_pricedb_lookup_nearest_in_time64(db, from, to, t);
 
@@ -2573,27 +2577,46 @@ direct_price_conversion (GNCPriceDB *db, const gnc_commodity *from,
     return retval;
 }
 
+static gnc_numeric
+get_nearest_price (GNCPriceDB *pdb,
+                   const gnc_commodity *orig_curr,
+                   const gnc_commodity *new_curr,
+                   const time64 t,
+                   gboolean before)
+{
+    gnc_numeric price;
+
+    if (gnc_commodity_equiv (orig_curr, new_curr))
+        return gnc_numeric_create (1, 1);
+
+    /* Look for a direct price. */
+    price = direct_price_conversion (pdb, orig_curr, new_curr, t, before);
+
+    /*
+     * no direct price found, try find a price in another currency
+     */
+    if (gnc_numeric_zero_p (price))
+        price = indirect_price_conversion (pdb, orig_curr, new_curr, t, before);
+
+    return gnc_numeric_reduce (price);
+}
+
+gnc_numeric
+gnc_pricedb_get_nearest_before_price (GNCPriceDB *pdb,
+                                      const gnc_commodity *orig_currency,
+                                      const gnc_commodity *new_currency,
+                                      const time64 t)
+{
+    return get_nearest_price (pdb, orig_currency, new_currency, t, TRUE);
+}
+
 gnc_numeric
 gnc_pricedb_get_nearest_price (GNCPriceDB *pdb,
                                const gnc_commodity *orig_currency,
                                const gnc_commodity *new_currency,
                                const time64 t)
 {
-    gnc_numeric price;
-
-    if (gnc_commodity_equiv (orig_currency, new_currency))
-        return gnc_numeric_create (1, 1);
-
-    /* Look for a direct price. */
-    price = direct_price_conversion (pdb, orig_currency, new_currency, t);
-
-    /*
-     * no direct price found, try find a price in another currency
-     */
-    if (gnc_numeric_zero_p (price))
-        price = indirect_price_conversion (pdb, orig_currency, new_currency, t);
-
-    return gnc_numeric_reduce (price);
+    return get_nearest_price (pdb, orig_currency, new_currency, t, FALSE);
 }
 
 gnc_numeric
@@ -2601,21 +2624,23 @@ gnc_pricedb_get_latest_price (GNCPriceDB *pdb,
                               const gnc_commodity *orig_currency,
                               const gnc_commodity *new_currency)
 {
-    return gnc_pricedb_get_nearest_price (pdb, orig_currency, new_currency, INT64_MAX);
+    return get_nearest_price (pdb, orig_currency, new_currency, INT64_MAX, FALSE);
 }
 
-static gnc_numeric convert_amount_at_date (GNCPriceDB *pdb,
-                                           gnc_numeric amount,
-                                           const gnc_commodity *orig_currency,
-                                           const gnc_commodity *new_currency,
-                                           const time64 t)
+static gnc_numeric
+convert_amount_at_date (GNCPriceDB *pdb,
+                        gnc_numeric amount,
+                        const gnc_commodity *orig_currency,
+                        const gnc_commodity *new_currency,
+                        const time64 t,
+                        gboolean before_date)
 {
     gnc_numeric price;
 
     if (gnc_numeric_zero_p (amount))
         return amount;
 
-    price = gnc_pricedb_get_nearest_price (pdb, orig_currency, new_currency, t);
+    price = get_nearest_price (pdb, orig_currency, new_currency, t, before_date);
 
     /* the price retrieved may be invalid. return zero. see 798015 */
     if (gnc_numeric_check (price))
@@ -2636,7 +2661,7 @@ gnc_pricedb_convert_balance_latest_price (GNCPriceDB *pdb,
                                           const gnc_commodity *new_currency)
 {
     return convert_amount_at_date
-        (pdb, balance, balance_currency, new_currency, INT64_MAX);
+        (pdb, balance, balance_currency, new_currency, INT64_MAX, FALSE);
 }
 
 gnc_numeric
@@ -2647,9 +2672,19 @@ gnc_pricedb_convert_balance_nearest_price_t64(GNCPriceDB *pdb,
                                               time64 t)
 {
     return convert_amount_at_date
-        (pdb, balance, balance_currency, new_currency, t);
+        (pdb, balance, balance_currency, new_currency, t, FALSE);
 }
 
+gnc_numeric
+gnc_pricedb_convert_balance_nearest_before_price_t64 (GNCPriceDB *pdb,
+                                                     gnc_numeric balance,
+                                                     const gnc_commodity *balance_currency,
+                                                     const gnc_commodity *new_currency,
+                                                     time64 t)
+{
+    return convert_amount_at_date
+        (pdb, balance, balance_currency, new_currency, t, TRUE);
+}
 
 /* ==================================================================== */
 /* gnc_pricedb_foreach_price infrastructure
